package org.eso.ohs.core.dbb.client;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.border.EmptyBorder;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbWidget abstract class,
 * defining a Check box widget.
 * <br>
 * @author Carlo Boarotto, ESO 20-Jul-1999
 * @version $Revision: 1.2 $
 */
public class DbbCheckBox extends DbbWidget implements ActionListener {

	/**
	 * Field defaultUnselected_
	 */
	private String defaultUnselected_ = null;

	/**
	 * Field defaultSelected_
	 */
	private String defaultSelected_ = null;

	/**
	 * Field cvsID_
	 */
	public static final String cvsID_ =
		"$Id: DbbCheckBox.java,v 1.2 2004/10/01 09:14:00 ddorigo Exp $";

	/**
	 * Field defaultIntSelected_
	 */
	int defaultIntSelected_ = 1;
	/**
	 * Field defaultIntUnselected_
	 */
	int defaultIntUnselected_ = 0;

	//----------------------------------------
	// Constructors
	//----------------------------------------

	/**
	 * Create a DbbCheckBox, with a specific label, associated with a
	 * specific chunk
	 *
	 * @param labeltext Label text to be displayed.
	 * @param chunk     DbbSqlChunk object.
	 */
	public DbbCheckBox(String labeltext, DbbSqlChunk chunk) {
		super(labeltext, chunk);
		setDefault("false");
		widget_ = new JCheckBox();
		widget_.setBorder(new EmptyBorder(new Insets(1, 1, 1, 1)));
		label_.setLabelFor(widget_);
		// Add listeners
		 ((JCheckBox) widget_).addActionListener(this);
	}

	/**
	 * Create a DbbCheckBox associated with a specific chunk.
	 *
	 * @param chunk DbbSqlChunk object.
	 */
	public DbbCheckBox(DbbSqlChunk chunk) {
		this(chunk.getLabel(), chunk);
	}

	//----------------------------------------
	// Public Methods
	//----------------------------------------

	/**
	 * Set the return default value when is selected and when is not selected.
	 * This value is return to the DbbSqlChunk only if the type
	 * of the DataType of the chunk is ancestor of Integer.class
	 * @param selected int
	 * @param unselected int
	 */
	public void setDefaultValuesIntChunk(int selected, int unselected) {
		defaultIntSelected_ = selected;
		defaultIntUnselected_ = unselected;
	}
	/**
		* Set the return default value when is selected. 
		* The value for unselected is the default
		* This value is return to the DbbSqlChunk only if the type
		* of the DataType of the chunk is ancestor of Integer.class
	 * @param selected int
		*/
	public void setDefaultValueForIntChunk(int selected) {
		defaultIntSelected_ = selected;
	}

	/**
	  * Set the return default value when is selected and when is not selected.
	  * This value is return to the DbbSqlChunk.
	  * @param selected String
	 * @param unselected String
	 */
	public void setDefaultValuesChunk(String selected, String unselected) {
		defaultSelected_ = selected;
		defaultUnselected_ = unselected;
	}

	/**
	 * Get the widget value.
	 *
	 * @return A String value: <code>true</code> if selected,
	 *         <code>false</code> otherwise
	 */
	public String get() {
		if (defaultSelected_ != null && defaultUnselected_ != null) {
			return standardDbWidgetGet();
		} else {
			return dbCheckBoxGet();
		}
	}

	/**
	 * Method standardDbWidgetGet
	 * @return String
	 */
	private String standardDbWidgetGet() {
		if (((JCheckBox) widget_).isSelected()) {
			return defaultSelected_;
		} else {
			return defaultUnselected_;
		}
	}

	/**
	 * Method dbCheckBoxGet
	 * @return String
	 */
	private String dbCheckBoxGet() {
	    if (sqlChunk_ != null) {
		Class c = sqlChunk_.getDataType();
		if (c.isAssignableFrom(Integer.class)) {
			if (((JCheckBox) widget_).isSelected()) {
				return Integer.toString(defaultIntSelected_);
			} else {
				return Integer.toString(defaultIntUnselected_);
			}
		} else if (
			c.isAssignableFrom(Boolean.class) && sqlChunk_.getUseNull()) {
			return "null";
		} else {
			return String.valueOf(((JCheckBox) widget_).isSelected());
		}
	    } else {
	        return "null";
	    }
	}

	/**
	 * Set the widget value. If the setDefaultValues is set, the get and set are consistent
	 *
	 * @param val Can be "1", "y", "yes", "true", "t", "on" (case meaning
	 *             TRUE. Any other value, including an empty string,
	 *             means FALSE
	 */

	public void set(String val) {
		String t = "n";
		if (defaultSelected_ != null && defaultUnselected_ != null) {
			if (val != null && val.equals(defaultSelected_)) {
				t = "y";
			}
		} else {
			t = val.toLowerCase();
		}
		boolean new_val =
			(t.equals("y")
				|| t.equals("yes")
				|| t.equals("1")
				|| t.equals("true")
				|| t.equals("t")
				|| t.equals("on"));
		((JCheckBox) widget_).setSelected(new_val);
		


	}

	/** @return A boolean value: <code>true</code> if it is <em>not</em>
	 *          selected, <code>false</code> otherwise.  * @return boolean
	 */
	public boolean isEmpty() {
		return !isSelected();
	}

	/** @return A boolean value: <code>true</code> if it is <em>not</em>
	   *          selected, <code>false</code> otherwise.  * @return boolean
	 */
	public boolean isSelected() {
		return (((JCheckBox) widget_).isSelected());
	}

	//----------------------------------------
	// Event methods
	//----------------------------------------

	/**
	 * Listener interface implementation.
	 * @param ev        Event triggering the action.
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent ev) {
		fireDbbWidgetAction();
	}
}
