package org.eso.ohs.core.docview.gui;

import java.awt.Color;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

/**
 * A GUI adaptor to interface between DocView and JTextComponent subclasses
 * By using a GuiAdaptor, DocView itself can work with a much wider range
 * of GUI components.
 * 
 * <p>
 * $Id: TextCompAdaptor.java,v 1.1 2004/02/17 12:38:00 tcanavan Exp $
 * <p>
 * @author Bob Kemp
 */
public class TextCompAdaptor implements GuiAdaptor, DocumentListener {

    /** GUI component whose value is to be read/updated */
    private JTextComponent tc_;

    /** Has the GUI component been modified?  Reset by DocView. */
    private boolean modified_;


    /**
     * Constructor
     * 
     * @param cBox	GUI component to be linked to
     */
    public TextCompAdaptor(JTextComponent tc) {
	tc_ = tc;
	tc.getDocument().addDocumentListener(this);
    }


    /**
     * Set the background colour of the underlying GUI component
     * 
     * @param bg	new value of background colour
     */
    public void setForeground(Color bg) {
	tc_.setForeground(bg);
    }


    /**
     * Enable the underlying GUI component
     * 
     * @param enabled	true if component should be enabled, false otherwise
     */
    public void setEnabled(boolean enabled) {
	tc_.setEnabled(enabled);
    }

    /**
     * Is the underlying GUI component enabled?
     * 
     * @return	true if component should be enabled, false otherwise
     */
    public boolean isEnabled() {
	return tc_.isEnabled();
    }

    /**
     * Set a new value for the GUI components value
     * 
     * @param newValue	new value represented as a String
     */
    public void setValue(String newValue) {
	tc_.setText(newValue == null ? "" : newValue.toString());
    }

    /**
     * Get the current value of the GUI component
     * 
     * @return	current value represented as a String
     */
    public String getValue() {
	return tc_.getText();
    }

    /**
     * Flag the component as having been modified (or not modified)
     * 
     * @param	new value of the flag
     */
    public void setModified(boolean modified) {
	modified_ = modified;
    }

    /**
     * Has the GUI component been modified?
     * 
     * @return	current value of the flag
     */
    public boolean isModified() {
	return modified_;
    }


    /** A listener method for changes to the underlying GUI component */
    public void insertUpdate(DocumentEvent e) {
	setModified(true);
    }

    /** A listener method for changes to the underlying GUI component */
    public void changedUpdate(DocumentEvent e) {
	setModified(true);
    }

    /** A listener method for changes to the underlying GUI component */
    public void removeUpdate(DocumentEvent e) {
	setModified(true);
    }
}

