/* $Id: Cycle.java,v 1.44 2005/11/09 15:51:11 ddorigo Exp $ */

package org.eso.ohs.dfs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.commons.collections.CollectionUtils;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.ObjectIOException;

/**
 * Observing Programme business object class. <br>
 * 
 * @author Carlo Boarotto, 10-Mar-2000
 */
/**
 * Class Cycle
 *
 * TODO Javadoc
 *
 * @version $Revision: 1.44 $
 * @author ddorigo, Mar 1, 2005
 */

// $Id: Cycle.java,v 1.44 2005/11/09 15:51:11 ddorigo Exp $
public class Cycle extends BusinessObject implements Comparable {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;

    //These are the properties of a cycle.
	private String cycleLabel_ = "";

	private int period_ = -1;

	private String description_ = "";

	private int step_id_ = -1;

	private Hashtable telInstTable_ = new Hashtable();

	//For the moment we assume that there is one interferometric telescope only
	// (VLTI).
	//private String[] interferometricTelescopes_ = {};
	private String[] countries_ = {};

	private String[] interferometricInstruments_ = {};

	private Hashtable baselines_ = new Hashtable();

	private Hashtable lambdacouples_ = new Hashtable();

	private String[] specialInstruments_ = {};

	private String[] validMonths_ = {};

	private String[] runcodeValues_ = {};

	private String[] altcodeValues_ = {};

	private String[] obsModeValues_ = {};

	private String[] moonList_ = {};

	private Integer[] periodList_ = {};

	private String[] seeingValues_ = {};

	private String[] skyTransparencyValues_ = {};

	private String[] subCategoryCodes_ = {};

	private String[] equinox_ = {};
	
	
	
	//----------------------new additions ----------------------
	private String[] proprietaryTimes_ = {};
	
	private String moonRequirements_;
	
	private String transparencyRequirements_;
	
	private String nations_;
	
	private String[] interfBaselines_ ={};
	
	private PeriodDependentData periodDate = new PeriodDependentData(); 
	
	//
	//Properties used internally.
	private int cycleId_ = -1;
	
	private String stepLabel_ = "";

	private CycleSteps steps_ = new CycleSteps();


//	private Hashtable telescopesInstruments_;

	//----------------------------------------
	// Constructor
	//----------------------------------------

	/**
	 * Create a Cycle and set its id.
	 */
	public Cycle() {
		super();
	}

	//----------------------------------------
	// Public Methods
	//----------------------------------------

    
    
	


	/**
	 * Set new value for cycle_label
	 * 
	 * @param cycle_label
	 *            the new value for the property
	 */
	public void setLabel(String cycleLabel) {
		cycleLabel_ = cycleLabel;
	}

	/**
	 * Get value for cycle_label
	 * 
	 * @return the current value of the property
	 */
	public String getLabel() {
		return cycleLabel_;
	}

	/**
	 * Set new value for period
	 * 
	 * @param period
	 *            the new value for the property
	 */
	public void setPeriod(int period) {
		period_ = period;
	}

	/**
	 * Get value for period
	 * 
	 * @return the current value of the property
	 */
	public int getPeriod() {
		return period_;
	}

	/**
	 * Set new value for description
	 * 
	 * @param description
	 *            the new value for the property
	 */
	public void setDescription(String description) {
		description_ = description;
	}

	/**
	 * Get value for description property
	 * 
	 * @return the current value of the property
	 */
	public String getDescription() {
		return description_;
	}
	
	public String getDeadline() {
		String result = description_;
		int a = description_.indexOf(":");
		int b = description_.indexOf("CEST");
		b = b + 4;
		result = result.substring(a+1, b);
		result = result.replaceAll("noon CEST", "CEST");
		return result;
	}

	/**
	 * Set value for step_id property
	 * 
	 * @param step_id
	 *            the new value of the property
	 */
	public void setStepId(int step_id) {
		step_id_ = step_id;
		initializeStepLabel(step_id_);
	}

	/**
	 * Get value for step_id property
	 * 
	 * @return the current value of the property
	 */
	public int getStepId() {
		return step_id_;
	}

	public String toString() {
		return cycleLabel_;
	}

	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}

	public int compareTo(Object obj) {
		int result;
		if (obj instanceof Cycle) {
			Cycle bo = (Cycle) obj;
			result = (int) (getId() - bo.getId());
		} else {
			throw new ClassCastException("Not a cycle obj");
		}
		return result;
	}

	public boolean equals(Object obj) {
		if (obj instanceof Cycle) {
			Cycle bo = (Cycle) obj;
			return bo.getId() == getId();
		}
		return false;
	}

	/**
	 * @see org.eso.ohs.dfs.StorableObject#postProecess()
	 */
	public void postProecess() {
	}

	/**
	 * @return Returns the stepLabel_.
	 */
	public String getStepLabel() {
		return stepLabel_;
	}

	/**
	 * @param stepLabel_
	 *            The stepLabel_ to set.
	 */
	public void setStepLabel(String stepLabel) {
		stepLabel_ = stepLabel;
		initializeStepId(stepLabel_);
	}

	/**
	 * @param stepLabel_
	 *            The stepLabel_ to set.
	 */
	private void initializeStepLabel(int stepId) {

		//System.out.println("Initialize the step label using the step id: " +
		// stepId);
		Integer stepIdInteger = new Integer(stepId);
		stepLabel_ = steps_.fromDatabaseData(stepIdInteger);
		//System.out.println("Initialize the step label using the step id: " +
		// stepId + " Step label for cycle " + this.getLabel() + " is: " +
		// getStepLabel());
	}

	private void initializeStepId(String stepLabel) {

		Integer stepIdInteger = (Integer) steps_.toDatabaseData(stepLabel);
		step_id_ = stepIdInteger.intValue();
	}

	/**
	 * @return Returns the baselines.
	 */
	public String[] getBaselines(String instrument) {

		String[] baselines = (String[])baselines_.get(instrument);
		if (baselines == null) {
			baselines = new String[0];
		}
		
		return baselines;
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	public void setBaselines(String instrument, String[] values) {

		baselines_.put(instrument, values);

	}

	/**
	 * @return Returs all the baselines
	 *  
	 */
	public String[] getAllBaselines() {

		Collection baselinesColl = baselines_.values();
		return getArrayFromCollection(baselinesColl);

	}
	
	//wrapper for cycledoc
	public String[] getAllbaselines(){
		return getAllBaselines();

	}

	/**
	 * @param baselines
	 *            The baselines to set.
	 */
	public void setBaselinesTable(Hashtable baselines) {
		baselines_ = baselines;
	}

	
	/**
	 * @return Hashtable
	 */
	public Hashtable getTelescopesInstrumentsTable() {

		return telInstTable_;
	}

	
	
	/**
	 * @param baselines
	 *            The baselines to set.
	 */
	public void setTelescopesInstrumentsTable(Hashtable instruments) {
		telInstTable_ = instruments;
	}

	
	
	

	/**
	 * @return Hashtable
	 */
	public Hashtable getBaselinesTable() {

		return baselines_;
	}
	
/*	public String[] getBaselines(){
		String[] result = new String[baselines_.size()];
			
			Enumeration e = baselines_.elements();
			int i = 0;
			while (e.hasMoreElements()){
				result[i] = (String) e.nextElement();
				i++;
			}
				
		return result;
	}*/
	
	
	
	
	/**
	 * @return Returns the equinox.
	 */
	public String[] getEquinox() {
		return equinox_;
	}

	/**
	 * @param equinox
	 *            The equinox to set.
	 */
	public void setEquinox(String[] equinox) {
		equinox_ = equinox;
	}

	/**
	 * @return Returns the instruments.
	 */
	public String[] getAllInstruments() {

		Collection instColl = telInstTable_.values();
		return getArrayFromCollection(instColl);
	}
	
	//wrapper for cycleDoc
	public String[] getAllinstruments(){
		
		return getAllInstruments();	
	}

	
	/**
	 * @param valuesCollection
	 * @return String[]
	 */
	private String[] getArrayFromCollection(Collection valuesCollection) {

		
		Vector valueVector = new Vector();
		Iterator valuesIterator = valuesCollection.iterator();
		while (valuesIterator.hasNext()) {
		
			String[] tmpValues = (String[])valuesIterator.next();
			for (int j=0; j<tmpValues.length; j++) {
					
					String stringValue = tmpValues[j];
					valueVector.add(stringValue);
				
			}
		}
		
		String[] values = (String[])valueVector.toArray(new String[valueVector.size()]);
		return values;
	}

	public String[] getInstruments(String telescope) {

		String[] instruments = (String[]) telInstTable_.get(telescope);	
		if (instruments == null) {
		    instruments = new String[0];
		}
		return instruments;
	}
	
	/*
	 * TODO hardcoded just to try the rest of the application
	 */
	public TableRuleBO getInstruments(){
		TableRuleBO result = new TableRuleBO();
		String col1 = "telescope";
		String col2 = "instruments";
		result.addKey(col1);
		result.addKey(col2);
		
		Hashtable hash = getTelescopesInstrumentsTable();
		Enumeration keys = hash.keys();
		while (keys.hasMoreElements()){
			String telescope = (String) keys.nextElement();
			String[] instrumentsList = (String[]) hash.get(telescope);
			StringBuffer str = new StringBuffer();
			for (int i=0; i < instrumentsList.length; i++){
				str.append(instrumentsList[i]);
				if (i < instrumentsList.length -1)
					str.append(", ");
			}
			String instruments = str.toString();
			HashMap map = new HashMap();
			map.put(col1, telescope);
			map.put(col2, instruments);
			result.addValueRow(map);
		}

		return result;
	}
	
	

	/**
	 * @param instruments
	 *            The instruments to set.
	 * @throws ObjectIOException
	 */
	public void setInstruments(String telescope, String[] instruments) throws ObjectIOException {
	    String[] tels = getAllTelescopes();
        Vector instrVec = new Vector();
	    CollectionUtils.addAll(instrVec,instruments);
	    for (int i = 0; i < tels.length; i++) {
	        String currTel = tels[i];
	        if (currTel.equals(telescope)) {
	         break;
	        }
	        String [] instrList = getInstruments(currTel);
	            for (int j = 0; j < instrList.length;j++ ) {
	                if (instrVec.contains(instrList[j])){
	                   String msg = "Attempting to add instrument "+instrList[j]+" in telescope "+telescope+
	                   "\n Instrument already present in telescope "+currTel;
	                    throw new ObjectIOException(msg);
	     
	                }
	           }
	       }
	    
		telInstTable_.put(telescope, instruments);
	}

	/**
	 * @return Returns the interferometricInstruments.
	 */
	public String[] getInterferometricInstruments() {
		return interferometricInstruments_;
	}

	/**
	 * @param interferometricInstruments
	 *            The interferometricInstruments to set.
	 */
	public void setInterferometricInstruments(
			String[] interferometricInstruments) {
		interferometricInstruments_ = interferometricInstruments;
	}

	/**
	 * @return Returns the baselines.
	 */
	public String[] getLambdacouples(String instrument) {
		
		String[] lambdacouples = (String[])lambdacouples_.get(instrument);
		if (lambdacouples == null) {
			lambdacouples = new String[0];
		}
		
		return lambdacouples;
		
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	public void setLambdacouples(String instrument, String[] values) {

		lambdacouples_.put(instrument, values);
	}

	/**
	 * @return Returs all the lambdacouples
	 *  
	 */
	public String[] getAllLambdacouples() {

		Collection lambdacouplesColl = lambdacouples_.values();
		return getArrayFromCollection(lambdacouplesColl);

	}

	/**
	 * @param lambdaCouples
	 *            The lambdaCouples to set.
	 */
	public void setLambdaCouplesTable(Hashtable lambdacouples) {
		lambdacouples_ = lambdacouples;
	}

	/**
	 * @return Returs the lambdacouples table
	 *  
	 */
	public Hashtable getLambdacouplesTable() {

		return lambdacouples_;

	}

	/**
	 * @return Returns the moonList.
	 */
	public String[] getMoonList() {
		return moonList_;
	}
	
	/**
	 * Wrapper method for cycle doc
	 * @return Returns the moonList.
	 */
	public String[] getMoonlist() {
		return getMoonList();
	}

	/**
	 * @param moonList
	 *            The moonList to set.
	 */
	public void setMoonList(String[] moonList) {
		moonList_ = moonList;
	}

	/**
	 * @return Returns the obsModeValues.
	 */
	public String[] getObsModeValues() {
		return obsModeValues_;
	}

	/**
	 * @param obsModeValues
	 *            The obsModeValues to set.
	 */
	public void setObsModeValues(String[] obsModeValues) {
		obsModeValues_ = obsModeValues;
	}

	/**
	 * @return Returns the periodList.
	 */
	public Integer[] getPeriodList() {
		return periodList_;
	}

	public String[] getPeriodListString() {

		Integer[] integerPeriodList = getPeriodList();
		int periodListLength = integerPeriodList.length;
		String[] stringPeriodList = new String[periodListLength];
		for (int i = 0; i < periodListLength; i++) {

			stringPeriodList[i] = integerPeriodList[i].toString();

		}
		return stringPeriodList;
	}

	//wrapper for cycleDoc
	public String[] getPeriodliststring(){
		return getPeriodListString();
	}
	/**
	 * @param periodList
	 *            The periodList to set.
	 */
	public void setPeriodList(Integer[] periodList) {
		periodList_ = periodList;
	}

	/**
	 * @return Returns the runcodeValues.
	 */
	public String[] getRuncodeValues() {
		return runcodeValues_;
	}

	/**
	 * @param runcodeValues
	 *            The runcodeValues to set.
	 */
	public void setRuncodeValues(String[] runcodeValues) {
		runcodeValues_ = runcodeValues;
	}

	/**
	 * @return Returns the seeingValues.
	 */
	public String[] getSeeingValues() {
		return seeingValues_;
	}
	/**
	 * Wrapper method for cycle doc
	 * @return Returns the moonList.
	 */
	public String[] getSeeingvalues() {
		return getSeeingValues();
	}
	/**
	 * Wrapper method for cycle doc
	 * @return Returns the proprietary Time.
	 */
	public String[] getProprietarytimes() {
		return getProprietaryTimes();
	}

	/**
	 * @param seeingValues
	 *            The seeingValues to set.
	 */
	public void setSeeingValues(String[] seeingValues) {
		seeingValues_ = seeingValues;
	}

	/**
	 * @return Returns the skyTransparencyValues.
	 */
	public String[] getSkyTransparencyValues() {
		return skyTransparencyValues_;
	}
	
	/**
	 * wrapper for cycledoc
	 * @return Returns the skyTransparencyValues.
	 */
					         
	public String[] getSkytransparencyvalues() {
		return getSkyTransparencyValues();
	}

	/**
	 * @param skyTransparencyValues
	 *            The skyTransparencyValues to set.
	 */
	public void setSkyTransparencyValues(String[] skyTransparencyValues) {
		skyTransparencyValues_ = skyTransparencyValues;
	}

	/**
	 * @return Returns the specialInstruments.
	 */
	public String[] getSpecialInstruments() {
		return specialInstruments_;
	}
	
	/**
	 * wrappet for cycleDoc
	 */
	public String[] getSpecialinstruments() {
		return getSpecialInstruments();
	}

	/**
	 * @param specialInstruments
	 *            The specialInstruments to set.
	 */
	public void setSpecialInstruments(String[] specialInstruments) {
		specialInstruments_ = specialInstruments;
	}

	/**
	 * @return Returns the subCategoryCodes.
	 */
	public String[] getSubCategoryCodes() {
		return subCategoryCodes_;
	}

	/**
	 * @param subCategoryCodes
	 *            The subCategoryCodes to set.
	 */
	public void setSubCategoryCodes(String[] subCategoryCodes) {
		subCategoryCodes_ = subCategoryCodes;
	}
	
	//wrapper for cycleDoc
	public String[] getSubcategorycodes() {
		return getSubCategoryCodes();
	}

	/**
	 * @return Returns the telescopes.
	 */
	public String[] getAllTelescopes() {

		Vector telescopeVector = new Vector();
		for (Enumeration telescopeEnum = telInstTable_.keys(); telescopeEnum.hasMoreElements();) {
			String telescope = (String) telescopeEnum.nextElement();
			if (!telescopeVector.contains(telescope))
			telescopeVector.add(telescope);		
		}
		String[] telescopes = (String[])telescopeVector.toArray(new String[telescopeVector.size()]);
		return telescopes;
	}

	/**
	 * @param telescopes
	 *            The telescopes to set.
	 */
	public void setTelescopes(String[] telescopes) {
	    if (telescopes == null) {
	    telescopes = new String[0];
	    }
		int telescopeArraySize = telescopes.length;
		
		for (int i = 0; i<telescopeArraySize; i++) {
			String telescope = telescopes[i];
			String[] instruments = getInstruments(telescope);
			telInstTable_.put(telescope, instruments);
		}
	}

	/**
	 * @return Returns the validMonths.
	 */
	public String[] getValidMonths() {
		return validMonths_;
	}

	/**
	 * @param validMonths
	 *            The validMonths to set.
	 */
	public void setValidMonths(String[] validMonths) {
		validMonths_ = validMonths;
	}

	/**
	 * @return Returns the countries.
	 */
	public String[] getCountries() {
		return countries_;
	}

	/**
	 * @param countries
	 *            The countries to set.
	 */
	public void setCountries(String[] countries) {
		countries_ = (String[]) countries;
	}
	

	/**
	 * @return Returns the altcodeValues.
	 */
	public String[] getAltcodeValues() {
		return altcodeValues_;
	}

	/**
	 * @param altcodeValues
	 *            The altcodeValues to set.
	 */
	public void setAltcodeValues(String[] altcodeValues) {
		altcodeValues_ = altcodeValues;
	}

    /**
     * @return Returns the cycleId_.
     */
    public int getCycleId() {
        return cycleId_;
    }
    /**
     * @param cycleId_ The cycleId_ to set.
     */
    public void setCycleId(int cycleId) {
        this.cycleId_ = cycleId;
    }

	public String[] getProprietaryTimes() {
		return proprietaryTimes_;
	}

	public void setProprietaryTimes(String[] proprietaryTimes) {
		this.proprietaryTimes_ = proprietaryTimes;
	}
	
//	Methods for period retrieval (current, future and past periods)
	public int getPeriodplus1(){
		int result = getPeriod() + 1;
		return result;
	}
	
	public int getPeriodplus2(){
		int result = getPeriod() + 2;
		return result;
	}
	
	public int getPeriodplus3(){
		int result = getPeriod() + 3;
		return result;
	}
	
	public int getPeriodplus4(){
		int result = getPeriod() + 4;
		return result;
	}
	
	public int getPeriodminus1(){
		int result = getPeriod() - 1;
		return result;
	}
	
	public int getPeriodminus2(){
		int result = getPeriod() - 2;
		return result;
	}
	
	public int getPeriodminus3(){
		int result = getPeriod() - 3;
		return result;
	}
	
	public int getPeriodminus4(){
		int result = getPeriod() - 4;
		return result;
	}
	
	//Methods for month code retrieval (without year)
	
	public String[] getMonthscode(){
		String[] result;
		ArrayList a = periodDate.getMonthCodeList(period_);
		result = (String[]) a.toArray(new String[a.size()]);
		return result;
	}
	
	public String getMonthcode1(){
		String result = (String) periodDate.getMonthCodeList(period_).get(0);
		return result;	
	}
	
	public String getMonthcode2(){
		String result = (String) periodDate.getMonthCodeList(period_).get(1);
		return result;	
	}
	
	public String getMonthcode3(){
		String result = (String) periodDate.getMonthCodeList(period_).get(2);
		return result;	
	}
	
	public String getMonthcode4(){
		String result = (String) periodDate.getMonthCodeList(period_).get(3);
		return result;	
	}
	
	public String getMonthcode5(){
		String result = (String) periodDate.getMonthCodeList(period_).get(4);
		return result;	
	}
	
	public String getMonthcode6(){
		String result = (String) periodDate.getMonthCodeList(period_).get(5);
		return result;	
	}
	
	public String getMonthcode7(){
		String result = (String) periodDate.getMonthCodeList(period_).get(6);
		return result;	
	}
	
	public String getMonthcode8(){
		String result = (String) periodDate.getMonthCodeList(period_).get(7);
		return result;	
	}
	public String getMonthcode9(){
		String result = (String) periodDate.getMonthCodeList(period_).get(8);
		return result;	
	}
	
	public String getMonthcode10(){
		String result = (String) periodDate.getMonthCodeList(period_).get(9);
		return result;	
	}
	
	public String getMonthcode11(){
		String result = (String) periodDate.getMonthCodeList(period_).get(10);
		return result;	
	}
	
	public String getMonthcode12(){
		String result = (String) periodDate.getMonthCodeList(period_).get(11);
		return result;	
	}
	
//	Methods for month+year code retrieval 
	public String getMonthyearcode1(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(0);
		return result;	
	}
	
	public String getMonthyearcode2(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(1);
		return result;	
	}
	
	public String getMonthyearcode3(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(2);
		return result;	
	}
	
	public String getMonthyearcode4(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(3);
		return result;	
	}
	
	public String getMonthyearcode5(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(4);
		return result;	
	}
	
	public String getMonthyearcode6(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(5);
		return result;	
	}
	
	public String getMonthyearcode7(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(6);
		return result;	
	}
	
	public String getMonthyearcode8(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(7);
		return result;	
	}
	public String getMonthyearcode9(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(8);
		return result;	
	}
	
	public String getMonthyearcode10(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(9);
		return result;	
	}
	
	public String getMonthyearcode11(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(10);
		return result;	
	}
	
	public String getMonthyearcode12(){
		String result = (String) periodDate.getMonthYearCodeList(period_).get(11);
		return result;	
	}
//	Methods for month full name retrieval (without year)
	public String getMonthfull1(){
		String result = (String) periodDate.getMonthFullList(period_).get(0);
		return result;	
	}
	
	public String getMonthfull2(){
		String result = (String) periodDate.getMonthFullList(period_).get(1);
		return result;	
	}
	
	public String getMonthfull3(){
		String result = (String) periodDate.getMonthFullList(period_).get(2);
		return result;	
	}
	
	public String getMonthfull4(){
		String result = (String) periodDate.getMonthFullList(period_).get(3);
		return result;	
	}
	
	public String getMonthfull5(){
		String result = (String) periodDate.getMonthFullList(period_).get(4);
		return result;	
	}
	
	public String getMonthfull6(){
		String result = (String) periodDate.getMonthFullList(period_).get(5);
		return result;	
	}
	
	public String getMonthfull7(){
		String result = (String) periodDate.getMonthFullList(period_).get(6);
		return result;	
	}
	
	public String getMonthfull8(){
		String result = (String) periodDate.getMonthFullList(period_).get(7);
		return result;	
	}
	public String getMonthfull9(){
		String result = (String) periodDate.getMonthFullList(period_).get(8);
		return result;	
	}
	
	public String getMonthfull10(){
		String result = (String) periodDate.getMonthFullList(period_).get(9);
		return result;	
	}
	
	public String getMonthfull11(){
		String result = (String) periodDate.getMonthFullList(period_).get(10);
		return result;	
	}
	
	public String getMonthfull12(){
		String result = (String) periodDate.getMonthFullList(period_).get(11);
		return result;	
	}
	
//	Methods for month+year full name retrieval 
	public String getMonthyearfull1(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(0);
		return result;	
	}
	
	public String getMonthyearfull2(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(1);
		return result;	
	}
	
	public String getMonthyearfull3(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(2);
		return result;	
	}
	
	public String getMonthyearfull4(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(3);
		return result;	
	}
	
	public String getMonthyearfull5(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(4);
		return result;	
	}
	
	public String getMonthyearfull6(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(5);
		return result;	
	}
	
	public String getMonthyearfull7(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(6);
		return result;	
	}
	
	public String getMonthyearfull8(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(7);
		return result;	
	}
	public String getMonthyearfull9(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(8);
		return result;	
	}
	
	public String getMonthyearfull10(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(9);
		return result;	
	}
	
	public String getMonthyearfull11(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(10);
		return result;	
	}
	
	public String getMonthyearfull12(){
		String result = (String) periodDate.getMonthYearFullList(period_).get(11);
		return result;	
	}
	
//	Methods for month+year code retrieval 
	public String getTimecritical1(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(0);
		return result;	
	}
	
	public String getTimecritical2(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(1);
		return result;	
	}
	
	public String getTimecritical3(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(2);
		return result;	
	}
	
	public String getTimecritical4(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(3);
		return result;	
	}
	
	public String getTimecritical5(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(4);
		return result;	
	}
	
	public String getTimecritical6(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(5);
		return result;	
	}
	
	public String getTimecritical7(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(6);
		return result;	
	}
	
	public String getTimecritical8(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(7);
		return result;	
	}
	public String getTimecritical9(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(8);
		return result;	
	}
	
	public String getTimecritical10(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(9);
		return result;	
	}
	
	public String getTimecritical11(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(10);
		return result;	
	}
	
	public String getTimecritical12(){
		String result = (String) periodDate.getMonthYearCodeList(period_, "-", 2).get(11);
		return result;	
	}
	
	public TableRuleBO getMoonRequirements(){
		TableRuleBO result = null;
		String[] columnList ={"code", "description"};
		result = stringToTableRuleBO(columnList, moonRequirements_);
		return result;
	}
	
	
	//wrapper method for cycleDoc
	public TableRuleBO getMoonrequirements(){
		return getMoonRequirements();
	}
	
	public void setMoonRequirements (String moonRequirements) {
		this.moonRequirements_ = moonRequirements;
	}
	
	
	public TableRuleBO getTransparencyRequirements(){
		TableRuleBO result = null;
		String[] columnList ={"code", "description"};
		result = stringToTableRuleBO(columnList, transparencyRequirements_);
		return result;
	}
	
	public void setTransparencyRequirements (String transparencyRequirements) {
		this.transparencyRequirements_ = transparencyRequirements;
	}
	
	public String getTransparencylist(){
		return this.transparencyRequirements_;
	}
	
	//wrapper method for cycleDoc
	public TableRuleBO getTransparencyrequirements(){
		
		
		return getTransparencyRequirements();
	}
	
	public TableRuleBO getNations(){
		TableRuleBO result = null;
		String[] columnList ={"code", "description"};
		result = stringToTableRuleBO(columnList, nations_);
		return result;
	}
	
	
	public void setNations (String nations) {
		this.nations_ = nations;
	}
	
	
	public String[] getTarget(){
		ArrayList tmp = periodDate.getTargetList(period_);
		String[] result = (String[]) tmp.toArray(new String[tmp.size()]);
		return result;	
	}
	
	public String[] getVltitarget(){
		ArrayList tmp = periodDate.getVLTITargetList(period_);
		String[] result = (String[]) tmp.toArray(new String[tmp.size()]);
		return result;	
	}
	
	public String[] getInterfBaselines() {
		return interfBaselines_;
	}
	
	//wrapper for cycleDoc feature
	public String[] getInterfbaselines() {
		return getInterfBaselines();
	}

	public void setInterfBaselines() {
		assert (interferometricInstruments_ !=  null): MsgManager.errInvariant("interforemetricInstruments is null");
		//assert (interferometricInstruments_.length != 0): MsgManager.errInvariant("interforemetricInstruments is empty");
		assert (baselines_ !=  null): MsgManager.errInvariant("baselines is null");
		//assert (baselines_.size() != 0): MsgManager.errInvariant("baselines is empty");
		
	
		ArrayList interfBaselines = new ArrayList();
		
		for (int i = 0; i < interferometricInstruments_.length; i++){
			String instrument = interferometricInstruments_[i];
			String[] baselines = getBaselines(interferometricInstruments_[i]);
			//Before to add the instrument title, let's add an empty line.
			//No empty line if it's the first instrument
			if (interfBaselines.size() != 0)
				interfBaselines.add("\n");
			interfBaselines.add(instrument);
			interfBaselines.add("");
			for (int j = 0; j < baselines.length; j++){
				interfBaselines.add(baselines[j]);
			}
		}
		//add an example for every instruments
	/*	interfBaselines.add("\n\nexamples:\n");
		String begin = "\\VLTITarget{F}{NGC 106}{-0.7}{-3.5}{10.6}{40}{";
		String end = "}{0.84}{-2.5}{6}";
		for (int i = 0; i < interferometricInstruments_.length; i++){
			String[] baselines = getBaselines(interferometricInstruments_[i]);
			interfBaselines.add(begin + baselines[0] + end);
		}*/
		this.interfBaselines_ = (String[]) interfBaselines.toArray(new String[interfBaselines.size()]);
	}
	
	
	private TableRuleBO stringToTableRuleBO(String[] columnList, String strTable) {
		assert (columnList != null): MsgManager.errPrecondition("ColumnList is null");
		assert (columnList.length != 0): MsgManager.errPrecondition("ColumnList is empty");
		assert (strTable != null): MsgManager.errPrecondition("strTable is null");
		assert (strTable.length()!= 0): MsgManager.errPrecondition("strTable is empty");
		TableRuleBO result = new TableRuleBO();
		
		for (int i = 0; i < columnList.length; i++){
			result.addKey(columnList[i]);	
		}
		
		/*
		 * let's parse the result string!
		 * We expect:
		 * 1) the string to have the format 
		 * 		(row1Item1,row1Item2,...)(row2Item1,row2Item2,...) etc.
		 * 2) for every row, the number of 
		 * 		items should match the number of items in the columList 
		 * 		array
		 * 3) the items are positional:
		 * 		if columnList = {"code", "description")
		 * 		then we expect that the first item of every row is the code
		 * 		while the second is the description.
		 */
		//lets normalize the string by cutting the first ( and the last )
		strTable = strTable.substring(1, strTable.length()-1);
		//lets tokenize it
		StringTokenizer rowTokenizer = new StringTokenizer(strTable, ")(");
		//lets populate every row using the tokens
		while (rowTokenizer.hasMoreTokens()){
			String rowToken = rowTokenizer.nextToken();
			StringTokenizer itemTokenizer = new StringTokenizer(rowToken, ";");
			//It's better to check with and if statement instead of using an assertion
			if (itemTokenizer.countTokens() != columnList.length) throw new IllegalArgumentException(MsgManager.errInvariant(columnList.length + " were expected instead of " + itemTokenizer.countTokens()));
			
			HashMap map = new HashMap();
			for (int i = 0; i < columnList.length; i++){	
				String itemToken = itemTokenizer.nextToken();
				map.put(columnList[i], itemToken);
			}	
			result.addValueRow(map);
		}
		return result;
	}

	
}