
package org.eso.ohs.dfs;

import org.eso.ohs.core.dbb.client.DbbDataType;
import org.eso.ohs.core.utilities.AssocList;
import org.eso.ohs.dfs.Mask;

/**
 * This is a helper class for the DBB which enables
 * the data stored in the database and the data displayed to
 * be automatically mapped to a display string
 * Example Status M for manufactured appears as MaskManufactured
 */

public class MaskStatusType implements DbbDataType {

    
    /** AssocList of strings and codes*/
    private static AssocList valList_ = null;
    
    /** Default list of strings */
    private final static String DEFAULT_ = "-----------";


    /** initialise mask status values*/
    private static void init() {
	if(valList_ == null)
	    valList_ = Mask.getStatusValues(); 
    }

    /**
     * Get the database values
     * @return array of string in the database
     */

    public static String[] getDatabaseValues() {
	init();
	return valList_.valuesAsStrings();
    }

    /**
     * Get the values to be displayed in the database
     * @return List of valid display strings
     *
     */

    public static String[] getDisplayValues() {
	init();
	return valList_.keys();
    }
    
    
    public int getDataTypeLength() {
	return DEFAULT_.length();
    }
    
    public Class getDataType() {
	return String.class;
    }
    
    public Class getDatabaseType() {
	return String.class;
    }


    /**
     * Convert the code in the database to a string to be presented
     * on the GUI
     * @return String for GUI presentation
     */
    
    public String fromDatabaseData(Object value) {
	init();
	String ret = "";
	String[] databaseValues_ = valList_.valuesAsStrings();
	String[] displayValues_ = valList_.keys();

	for(int i=0; i<databaseValues_.length; i++) {
	    if(databaseValues_[i].equals(value)) {
		ret = displayValues_[i];
		break;
	    }
	}

	return ret;
    }
    
    /**
     * Convert string to database data
     * @param value display data
     * @return database data
     */

    public Object toDatabaseData(String value) {
	init();	
	return valList_.valueFor(value);
    }
}
