/* $Id: ObsProgramme.java,v 1.19 2005/02/04 15:16:30 ddorigo Exp $ */

package org.eso.ohs.dfs;

import java.util.Date;

import org.apache.log4j.Logger;

/**
 * Observing Programme business object class.
 * <br>
 * 
 * @author Carlo Boarotto, 10-Mar-2000
 * @version $Revision: 1.19 $
 */
public class ObsProgramme extends BusinessObject {

	private boolean opticon_ = false;
    private boolean xmm_ = false;
    private boolean rrm_ = false;
    
	private int keyprog_;

	private static final String rcsid =
		"$Id: ObsProgramme.java,v 1.19 2005/02/04 15:16:30 ddorigo Exp $";

	private static Logger stdlog_ = Logger.getLogger(ObsProgramme.class);

	private int period_; //Observing period (ex: 65). 
	private int tableId_; //id in db table

	/** cycle_label. */
	private String cycleLabel_ = "";

	private int cycle_; //Observing cycle id . 
	private String panel_; //Proposal panel (or category).
	private int number_; //The message number of the proposal  
	private String title_ = ""; //Proposal title.
	private int attachments_; //The number of figures 
	private String proposalID_ = ""; // Proposal ID (ex: D-0002).
	private String proposalCode_ = ""; // Proposal ID (ex: 069.D-0002).
	private String postAddr_ = ""; // postal addr not used after period 62
	private String emailAddr_ = "";
	// enail addr of the submitter of the proposal
	private Date inDate_; //The receiving date
	private Date procDate_; //  The processing date 
	private boolean ignored_ = false; //  Ignored flag.
	private int printed_ = -1; //  printed flag.
	private int obsRunsNum_; //The number of runs 
	private int gto_; //  the gto field.
	private String subpanel_; //Proposal subcategory.
	private int opcSubpanel_; //Proposal opc subpanel.

	//----------------------------------------
	// Constructor
	//----------------------------------------

	/**
	 * Create a ObsProgramme and set its id.
	 */
	public ObsProgramme() {
		super();
	}

	//----------------------------------------
	// Public Methods
	//----------------------------------------

	/**
	 * Get value for period_ property
	 *
	 * @return the current value of the property
	 */
	public int getPeriod() {
		return period_;
	}
	
	/**
		 * Get value for opticon_ property
		 *
		 * @return the current value of the property
		 */
		public boolean isOpticon() {
			return opticon_;
		}

		/**
		 * Get value for xmm_ property
		 *
		 * @return the current value of the property
		 */
		public boolean isXmm() {
			return xmm_;
		}
		
		
		/**
		 * Get value for rrm_ property
		 *
		 * @return the current value of the property
		 */
		public boolean isRrm() {
			return rrm_;
		}
		
		
		
	/**
	 * Set new value for period_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setPeriod(int newValue) {
		period_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	  * Get value for id property
	  *
	  * @return the current value of the property
	  */
	public int getTableId() {
		return tableId_;
	}

	/**
	 * Set new value for id property
	 *
	 * @param newValue the new value for the property
	 */
	public void setTableId(int newValue) {
		tableId_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	   * Get value for cycleId_ property
	   *
	   * @return the current value of the property
	   */
	public String getCycleLabel() {

		return cycleLabel_;
	}

	/**
	 * Set new value for cycleId_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setCycleLabel(String newValue) {
		cycleLabel_ = newValue;
	}

	/**
	  * Get value for  property
	  *
	  * @return the current value of the property
	  */
	public int getCycle() {
		return cycle_;
	}

	/**
	 * Set new value for  property
	 *
	 * @param newValue the new value for the property
	 */
	public void setCycle(int newValue) {
		cycle_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for panel_ property
	 *
	 * @return the current value of the property
	 */
	public String getPanel() {
		return panel_;
	}

	/**
	 * Set new value for panel_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setPanel(String newValue) {

		panel_ = newValue;
		firePropertyChangeEvent();
	}

    /**
     * Set new value for panel_ property
     *
     * @param newValue the new value for the property
     */
    public void setPanelAndPropCode(String newValue) {

        panel_ = newValue;
        StringBuffer proposalCode = new StringBuffer(getProposalCode());
        proposalCode.setCharAt(4, panel_.charAt(0));
        setProposalCode(proposalCode.toString());

        StringBuffer proposalId = new StringBuffer(getProposalID());
        proposalId.setCharAt(0, panel_.charAt(0));
        setProposalID(proposalId.toString());

        firePropertyChangeEvent();
    }
	/**
	 * Get value for Number_ property
	 *
	 * @return the current value of the property
	 */
	public int getNumber() {
		return number_;
	}

	/**
	 *
	 * @param newValue the new value for the property
	 */
	public void setNumber(int newValue) {
		number_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for title_ property
	 *
	 * @return the current value of the property
	 */
	public String getTitle() {
		return title_;
	}

	/**
	 * Set new value for title_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setTitle(String newValue) {
		title_ = newValue;
		firePropertyChangeEvent();
	}


	/**
	 * Get value for attachments_ property
	 *
	 * @return the current value of the property
	 */
	public int getAttachments() {
		return attachments_;
	}

	/**
	 *
	 * @param newValue the new value for the property
	 */
	public void setAttachments(int newValue) {
		attachments_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	   * Get value for proposalID_ property
	   *
	   * @return the current value of the property
	   */
	public String getProposalID() {
		return proposalID_;
	}

	/**
	 * Set new value for proposalID_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setProposalID(String newValue) {
		proposalID_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	   * Get value for proposalCode_ property
	   *
	   * @return the current value of the property
	   */
	public String getProposalCode() {
		return proposalCode_;
	}

	/**
	 * Set new value for proposalCode_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setProposalCode(String newValue) {
		proposalCode_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value from derived from other properties
	 *
	 * @return the current value of the property
	 */
	public String getCompleteProposalID() {
		return new String(period_ + "." + proposalID_);
	}

	/**
	    * Get property value
	    *
	    * @return the current value of the property
	    */
	public String getPostAddr() {
		return postAddr_;
	}

	/**
	 * Set new  property value
	 * @param newValue the new value for the property
	 */
	public void setPostAddr(String newValue) {
		postAddr_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	    * Get property value
	    *
	    * @return the current value of the property
	    */
	public String getEmailAddr() {
		return emailAddr_;
	}

	/**
	 * Set new  property value
	 * @param newValue the new value for the property
	 */
	public void setEmailAddr(String newValue) {
		emailAddr_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get property value
	 *
	 * @return the current value of the property
	 */
	public Date getInDate() {
		return inDate_;
	}

	/**
	 * Set new  property value
	 * @param newValue the new value for the property
	 */
	public void setInDate(Date newValue) {
		inDate_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get property value
	 *
	 * @return the current value of the property
	 */
	public Date getProcDate() {
		return procDate_;
	}

	/**
	 * Set new  property value
	 * @param newValue the new value for the property
	 */
	public void setProcDate(Date newValue) {
		procDate_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for ignored_ property
	 *
	 * @return the current value of the property
	 */
	public boolean getIgnored() {
		return ignored_;
	}

	/**
	 * Set(rs.getDate()); new value for ignored_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setIgnored(boolean newValue) {
		ignored_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get property  value 
	 *
	 * @return the current value of the property
	 */
	public int getPrinted() {
		return printed_;
	}

	/**
	 * Set new property value 
	 *
	 * @param newValue the new value for the property
	 */
	public void setPrinted(int newValue) {
		printed_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get property  value 
	 *
	 * @return the current value of the property
	 */
	public int getObsRunsNum() {
		return obsRunsNum_;
	}

	/**
	 * Set new property value 
	 *
	 * @param newValue the new value for the property
	 */
	public void setObsRunsNum(int newValue) {
		obsRunsNum_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	    * Get property  value 
	    *
	    * @return the current value of the property
	    */
	public boolean isLarge() {
		if (gto_ == 4) {
			return true;
		}
		return false;
	}

	/**
	 * Get property  value 
	 *
	 * @return the current value of the property
	 */
	public int getKeyprog() {
		return keyprog_;
	}

	/**
	 * Set new property value 
	 *
	 * @param newValue the new value for the property
	 */
	public void setKeyprog(int newValue) {
		keyprog_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get property  value 
	 *
	 * @return the current value of the property
	 */
	public int getGto() {
		return gto_;
	}

	/**
	 * Set new property value 
	 * this would update accordingly  the proposal code
	 *
	 * @param newValue the new value for the property
	 */
	public void setGto(int newValue) {
		gto_ = newValue;
		firePropertyChangeEvent();
	}
    
    /**
     * Set new property value 
     * this would update accordingly  the proposal code
     *
     * @param newValue the new value for the property
     */
    public void setGtoAndPropCode(int newValue) {
        gto_ = newValue;
        StringBuffer proposalCode = new StringBuffer(getProposalCode());

        if (gto_ == 4) { // large program
            proposalCode.setCharAt(0, '1');
        } else if (gto_ == 0 || gto_ == 1 || gto_ == 3) { // normal, gto, too
            proposalCode.setCharAt(0, '0');
        } else if (gto_ == 2) { // ddt
            proposalCode.setCharAt(0, '2');
        }
        setProposalCode(proposalCode.toString());
        firePropertyChangeEvent();
    }


	/**
	 * Get value for opcSubpanel_ property
	 *
	 * @return the current value of the property
	 */
	public String getSubpanel() {
		return subpanel_;
	}

	/**
	 * Set new value for opcSubpanel_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setSubpanel(String newValue) {
		subpanel_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for opcSubpanel_ property
	 *
	 * @return the current value of the property
	 */
	public int getOpcSubpanel() {
		return opcSubpanel_;
	}

	/**
	 * Set new value for opcSubpanel_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setOpcSubpanel(int newValue) {
		opcSubpanel_ = newValue;
		firePropertyChangeEvent();
	}

	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }

	public void setOpticon(boolean opticon_) {
		this.opticon_ = opticon_;
	}

	public void setXmm(boolean xmm_) {
		this.xmm_ = xmm_;
	}
	
	public void setRrm(boolean rrm_) {
		this.rrm_ = rrm_;
	}
}
