package org.eso.ohs.dfs;

/**
 * A QueueItem object contains data regarding a scheduled 
 * ObservationBlock
 * 
 * @author Giorgio Passera <ESO\DMD> <gpassera@eso.org>
 */

public class QueueItem {
    private int startTime = 0; //expected time/date for start of OB execution
    private int endTime = 0; //expected time/date for end of OB execution
    private long obId;
    private int sequence = -1;
    private long schId = -1;

    private String type_;
    private String rank = ""; 

    /**Default constructor
     *@param odId the ObservationBlockId
     *@param startTime expected time/date for start of OB execution 
     *      (stored as an integer in Unix date format)
     *@param endTime expected time/date for end of OB execution 
     *      (stored as an integer in unix date format)
    */

    public QueueItem(long obId, long schId, int sequence, String type) {
        this(obId,schId,sequence);
        type_ = type;
    }
    
    
    public QueueItem(long obId, long schId, int sequence, String type,String rank) {
       this(obId,schId,sequence,type);
        this.rank = rank;
    }

    public QueueItem(long obId, long schId, int sequence) {
        this.obId = obId;
        this.schId = schId;
        this.sequence = sequence;
    }

    public QueueItem() {
    }

    /**Sets the time/date for start of OB excution*/
    public void setStartTime(int time) {
        startTime = time;
    }

    /**Sets the time/date for end of OB excution*/
    public void setEndTime(int time) {
        endTime = time;
    }

    /**Gets the time/date for start of excution*/
    public int getStartTime() {
        return startTime;
    }

    /**Gets the time/date for end of excution*/
    public int getEndTime() {
        return endTime;
    }

    /**Gets the ObservationBlock id*/
    public long getObId() {
        return obId;
    }
    public void setObId(long obId) {
        this.obId = obId;
    }

    public long getSchId() {
        return schId;
    }

    public void setSchId(long schId) {
        this.schId = schId;
    }
    public void setSequence(int sequence) {
        this.sequence = sequence;
    }

    public int getSequence() {
        return sequence;
    }
    public void setRank(String rank) {
        this.rank = rank;
    }

    public String getRank() {
        return rank;
    }

    public String toString() {
        return "OB ID " + obId;
    }

    public String getType() {
        return type_;
    }

    public void setType(String type) {
        type_ = type;
    }

}
