package org.eso.ohs.dfs;


public class SkyTransparency {

    private String sky_ = null;

    static final String PHOTOMETRIC_VIEW = "Photometric";

    static final String CLEAR_VIEW = "Clear";

    static final String THIN_CIRRUS_VIEW = "Variable, thin cirrus";

    static final String THICK_CIRRUS_VIEW = "Variable, thick cirrus";

    /**
     * Legal values for the SkyTransparency property
     */
    public static final String[] validSkyTransparencyViewArray =
    { SkyTransparency.PHOTOMETRIC_VIEW, SkyTransparency.CLEAR_VIEW,
            SkyTransparency.THIN_CIRRUS_VIEW, SkyTransparency.THICK_CIRRUS_VIEW };

    public static final String PHOTOMETRIC = "1PHO";

    public static final String CLEAR = "2CLR";

    public static final String THIN_CLOUDS = "3THN";

    public static final String THIK_CLOUDS = "4THK";

    /**
     * Legal values for the SkyTransparency property in the database
     */
    public static final String[] skyTransparencyDbRange =
    { PHOTOMETRIC, CLEAR, THIN_CLOUDS, THIK_CLOUDS };

    /**
     * must be between 0.2 and 2.0
     * 
     * @param seeing
     *            void
     */
    public static boolean isValidSky(String skyTransp) {
        boolean result = false;
        if (getIndexSky(skyTransp)>=0) {
            result = true;
        }
        return result;
    }

    /**
     * @param skyTransp
     * @param result
     * @return boolean
     */
    private static int getIndexSky(String skyTransp) {
        int index = -1;
        for (int i = 0; i < skyTransparencyDbRange.length; i++) {
            if (skyTransparencyDbRange[i].equals(skyTransp)) {
                index = i;
                break;
            }
        }
        return index;
    }

    public SkyTransparency(String sky) {
        if (isValidSky(sky)) {
            sky_ = sky;
        } else {
            throw new IllegalArgumentException(
                    "The sky transparency is not valid:" + sky);
        }

    }

    /**
     * @return String No duplication: the member is passed by reference (it is
     *         immutable).
     */
    public String get() {
        return sky_;
    }

    /**
     * Returns the database column value corresponding to a
     * SkyTransparency value.
     * 
     * @return SkyTransparency value
     */
    public static String toDbSkyTransparency(String viewValue) {
        for (int i = 0; i < validSkyTransparencyViewArray.length; i++) {
            if (validSkyTransparencyViewArray[i].equals(viewValue))
                return skyTransparencyDbRange[i];
        }
        return null;
    }

    public static String userViewFromDb(String sky) {
        String userValue = "";
        int index = getIndexSky(sky);
        if (index>=0) {
         userValue = validSkyTransparencyViewArray[index];
        }
        return userValue;
    }
}
