package org.eso.ohs.dfs;

import java.beans.PropertyChangeListener;
import java.io.Serializable;

/**
 * All objects that are to be stored on external media must implement
 * this interface.  Every business object has a unique Id which is
 * unique throughout the whole system.
 *
 * <p>
 * $Id: StorableObject.java,v 1.5 2003/12/07 12:49:31 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public interface StorableObject extends Serializable {

    /**
     * Returns a unique key for a given business object
     * @return Unique id of the Storable Object.
     */
    public long getId();

    /**
     * Initialises the key for a storable object.
     * 
     * @param newValue	    Unique key value
     */
    public void setId(long newValue);

    /**
     * Returns the name for a given business object
     * @return		    title of the Storable Object.
     */
    public String getName();
    
    
    
    /**
     * One may need to execute extra code
     *  after an object is loaded.
     */

    public void postProecess();
    

    /**
     * Initialises the name for a storable object
     * 
     * @param newValue	    title for this object
     */
    public void setName(String newValue);

    /**
     * Add listener
     * 
     * @param l PropertyChangeListener
     */
    public void addPropertyChangeListener(PropertyChangeListener l);


    /**
     * Remove listener
     * 
     * @param l PropertyChangeListener
     */
    public void removePropertyChangeListener(PropertyChangeListener l);

}



