package org.eso.ohs.persistence.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eso.ohs.dfs.Account;
import org.eso.ohs.dfs.AccountProfile;
import org.eso.ohs.dfs.Country;
import org.eso.ohs.dfs.Institution;
import org.eso.ohs.dfs.Roles;
import org.eso.ohs.dfs.Title;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateCallback;
import org.springframework.orm.hibernate3.HibernateTemplate;

public class AccountDAOHibernate implements AccountDAO {
    
    private HibernateTemplate hibernateTemplate; 

    public AccountDAOHibernate() {
        hibernateTemplate = new HibernateTemplate();
    }
    
    public Account getAccount(Integer id) throws DataAccessException {
       return (Account)hibernateTemplate.load(Account.class,id); 
    }

    public void create(Account account) throws DataAccessException {
    	
    	
    	System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>> Create new account: " + account.getAccountName());
        Collection c = account.getAccountProfile();
        Iterator iterator = c.iterator();
        while(iterator.hasNext()) {
            AccountProfile profile = (AccountProfile) iterator.next();
            setTitle(profile);
            setInstitution(profile);
          
        }
        System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>> Set role");
        setRoles(account);
        hibernateTemplate.saveOrUpdate(account);
    }
    
    
    

    public void updateAccount(Account account) throws DataAccessException {
    	hibernateTemplate.update(account);
    }

    /**
     * @return Returns the hibernateTemplate.
     */
    public HibernateTemplate getHibernateTemplate() throws DataAccessException {
        return hibernateTemplate;
    }

    /**
     * @param hibernateTemplate The hibernateTemplate to set.
     */
    public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
        this.hibernateTemplate = hibernateTemplate;
    }

    public Account getFullAccountDatails(String userName) throws IllegalArgumentException,DataAccessException {
        String [] tokens = new String[1];
        tokens[0] = userName;  
        List l = hibernateTemplate.find("from Account Account where Account.accountName = ?",tokens);
        if(l.size() > 1) {
            throw new IllegalArgumentException("Duplicate username in system");            
        }
        Account user = (Account)l.get(0);
        return user;
    }
    
    public Account getAccount(String userName) throws DataAccessException {    	   
        String [] tokens = new String[1];
        tokens[0] = userName;  
        List l = hibernateTemplate.find("from Account Account where Account.accountName = ?",tokens);
        if(l.size() > 1) {
            throw new IllegalArgumentException("Duplicate username in system");            
        }
        Account user = (Account)l.get(0);
        return user;
    }
    
    
    /**
     * This gets the full class hierarchy for a given hierarchy  
     * including the profile list. 
	 */	
	public Account getUserAndProfiles(final String name) throws DataAccessException {
	   HibernateCallback callback = new HibernateCallback(){

	    public Object doInHibernate(Session session) throws HibernateException {
			Account account =null;
			String query = "from Account Account where Account.accountName=?";
			Query queryObject = session.createQuery(query);
			queryObject.setParameter(0, name);
			account = (Account) queryObject.list().get(0);
			account.getAccountProfile().size();
			return account;
			}			
		};
		
		return (Account) getHibernateTemplate().execute(callback);
	}
    
    private void setTitle(AccountProfile profile) {
        String [] tokens = new String[1];
        tokens[0] = profile.getTitleId().getTitle();  
        List l = hibernateTemplate.find("from Title Title where Title.title = ?",tokens);
        if(l.size() == 0 ) {
            hibernateTemplate.save(profile.getTitleId());                             
        }
        l = hibernateTemplate.find("from Title Title where Title.title = ?",tokens);
        Title title = (Title)l.get(0);
        profile.setTitleId(title);
    }
    
    private void setRoles(Account account) {
        String [] tokens = new String[1];
        Collection roles = account.getRolesMany();
        System.out.println("...................................... Size of roles: " + roles.size());
        Collection dbRoles = new ArrayList();
        Iterator itroles = roles.iterator();
        while(itroles.hasNext()) {
            Roles role = (Roles)itroles.next();
            tokens[0] = role.getRoleName();
            List l = hibernateTemplate.find("from Roles Roles where Roles.roleName = ?",tokens);
            if(l.size() <= 0) {
                throw new IllegalArgumentException("Roles does not exist on database " + tokens[0]);
            }
            Roles dbRole = (Roles)l.get(0);
            System.out.println("==================================>>> Db role: " + dbRole.getRoleId());
            dbRoles.add(dbRole);            
        }
        account.setRolesMany(dbRoles);
    }
    

    
    private void setCountry(Institution institution) {
        String [] tokens = new String[1];
        tokens[0] = institution.getCountry().getCountry().trim();  
        List l = hibernateTemplate.find("from Country Country where Country.country = ?",tokens);
        System.out.println("---------------------------------- Country is: " + tokens[0]);
        Country country = (Country)l.get(0);
        institution.setCountry(country);
    }
    
    
    private void setInstitution(AccountProfile profile) {
    	Institution institution = profile.getAddress().getInstitution();
        Object [] tokens = new Object [2];
        tokens[0] = institution.getInstitutionName();
        tokens[1] = institution.getCountry().getCountry();
        List l = hibernateTemplate.find("from Institution Institution where Institution.institutionName = ? AND Institution.country.country = ?",tokens);
        if(l.size() == 0 ) {
        	institution = profile.getAddress().getInstitution();
        	setCountry(institution);
            //hibernateTemplate.save(profile.getAddress().getInstitution());                             
        } else {
            Institution inst = (Institution)l.get(0);
            profile.getAddress().setInstitution(inst);
        }
    }


    public void delete(Account account) throws DataAccessException {
        hibernateTemplate.delete(account);
    }
 
    public void changePassword(Account account) throws DataAccessException {
    	hibernateTemplate.update(account);
    }

	public boolean accountExists(String userName)  throws DataAccessException {
        String [] tokens = new String[1];
        tokens[0] = userName; 
        List l = hibernateTemplate.find("from Account Account where Account.accountName = ?",tokens);
        if(l.size() > 0 ) {
        	return true;
        }
        return false;
	}

	
	
	public AccountProfile getDefaultProfile(final String userName) throws DataAccessException {
	   HibernateCallback callback = new HibernateCallback() {
	    public Object doInHibernate(Session session) throws HibernateException {	    	
	    	 Query q = session.createQuery(
	                 "from Account as account " +
	                 "inner join account.accountProfile as defaultProfileItem " +
	                 "where defaultProfileItem.defaultProfile = :defProfile " +
	                 " AND account.accountName = :accName"
	             );
	    	 q.setBoolean("defProfile",true);
	    	 q.setString("accName",userName);
        	List filterAccProfile = q.list();
	    	Iterator filterAccIterator = filterAccProfile.iterator();
            Object [] pair = (Object [])filterAccIterator.next();
            AccountProfile accountProfile = (AccountProfile)pair[1]; 
	    	return accountProfile;
	    }
		};		
		return (AccountProfile) getHibernateTemplate().execute(callback);         		
	}
	
	public AccountProfile getDefaultProfile(final Account account) throws DataAccessException {
		   HibernateCallback callback = new HibernateCallback() {
		    public Object doInHibernate(Session session) throws HibernateException {
		    	Account accountTemp = getUserAndProfiles(account.getAccountName()); 
	        	Collection filterAccProfile = session.createFilter(accountTemp.getAccountProfile(),
	            "where this.defaultProfile = ?").setParameter(0,Boolean.TRUE).list();
		    	//AccountProfile accountProfile = (AccountProfile)filterAccProfile.get(0);
		    	return null;
		    }
			};		
			return (AccountProfile) getHibernateTemplate().execute(callback);
	         		
		}

	public void updateProfile(AccountProfile accProfile) throws DataAccessException {
        setTitle(accProfile);
        setInstitution(accProfile);
		hibernateTemplate.update(accProfile);
		
	}

	public Account getAccountRoles(final String userName) throws DataAccessException {
		   HibernateCallback callback = new HibernateCallback(){

			    public Object doInHibernate(Session session) throws HibernateException {
					Account account =null;
					String query = "from Account Account where Account.accountName=?";
					Query queryObject = session.createQuery(query);
					queryObject.setParameter(0,userName);
					account = (Account) queryObject.list().get(0);
					account.getRolesMany().size();
					return account;
					}			
				};
				
				return (Account) getHibernateTemplate().execute(callback);
	
	}
}
