package org.eso.ohs.persistence.dbase.phase1;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseHandlerBaseTG;

/**
 * I/O class for Targets specified as part of phase 1.
 *
 * <p>
 * $Id: DbaseHandlerTG1.java,v 1.1 2005/03/30 09:56:11 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerTG1 extends  DbaseHandlerBaseTG {

   /** Standard log channel */ 
   private static Logger stdlog_ =
       Logger.getLogger( DbaseHandlerTG1.class );


    private final static int MAX_PHASE1_TARGETS = 1000;

    /**
     * Managed class
     * @return Target class
     */
    public Class getManagedClass() {
	return Target.class;
    }

    public static int tableIndexFromId(long id) {
	return (int) (id % MAX_PHASE1_TARGETS);
    }

    public static long programmeIdFromId(long id) {
	return id / MAX_PHASE1_TARGETS;
    }

    public static long idForPhase1Target(long programmeId, int index) {
	return programmeId*MAX_PHASE1_TARGETS + index;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
			throws ObjectNotFoundException,
				IOException, SQLException {
	ResultSet rs = null;
	Target tg = (Target)newInstance();
	UnpackedStorableObject uso = new UnpackedStorableObject(tg);

	String sql = "SELECT * FROM targets"
			+ " WHERE id =" + tableIndexFromId(id)
			+ " AND   programme_id =" + programmeIdFromId(id);

	stdlog_.debug(sql);
	rs = stmt.executeQuery(sql);

	if ( ! rs.next()) { // Only one with this id !!
	    throw new ObjectNotFoundException("Object not found in database");
	}

	tg.setId(id);
	tg.setName(rs.getString("item_name"));
	tg.setRA(rs.getInt("r_a")/DbaseHandlerBaseTG.DEGREES_PER_HOUR);
	tg.setDeclination(rs.getInt("dec"));
	// tg.setMag(rs.getInt("magnitude"));
	tg.setUserComments(rs.getString("comments"));

	tg.setEquinox("J2000");		// Hardwired values used by PHRS
	tg.setEpoch(2000);		// Hardwired values used by PHRS

	rs.close();

	return uso;
    }


    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws ObjectIOException {

	throw new ObjectIOException("Operation not supported for this object");

    }


    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws ObjectIOException {

	throw new ObjectIOException("Operation not supported for this object");

    }


    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     */
    public void delete(Statement stmt, long id)
			throws ObjectIOException {
	throw new ObjectIOException("Operation not supported for this object");
    }

}


