package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
/**
 * I/O class for Schedules stored in the database.
 *
 
 * @author Giorgio Passera <ESO\DMD> <gpassera@eso.org>
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerMaskQueue extends DbaseHandlerQueue {

    private static String obrepDbName_ = Config.getCfg().getDbName(
					       ObservationBlock.getSuffix());



    public static String EXEC_SEQ  =  "exec_sequences";
    public static String TMP_EXEC_SEQ  =  "#tmp_exec_sequences";
   
    /**
     * Refers to the appropriate DFS class
     * @return Queue class
     */

    public Class getManagedClass() {
	return MaskQueue.class;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * @exception ObjectIOException IO Eror occured while reading from the database
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
			throws ObjectNotFoundException,
				ObjectIOException,
   				IOException, SQLException {
	ResultSet rs = null;
	MaskQueue  aQueue = (MaskQueue)newInstance();
	UnpackedStorableObject obj = new UnpackedStorableObject(aQueue);

	String sql = "SELECT * FROM " + obrepDbName_ + ".." + SCHEDULES
			+ " WHERE " + obrepDbName_ + ".." + SCHEDULES + ".sch_id = " + id;
        /*
	if ( ! Config.getCfg().isOperationsModeId(userId_) ) {
	    sql = sql + " AND " + obrepDbName + "..schedules.user_id=" + userId_;
	}
        */
	
	System.out.println("CLASS DbaseHandlerQueue - read: sql = " + sql);

	rs = stmt.executeQuery(sql);

	if ( ! rs.next()) { // Only one with this id !!
	    throw new ObjectNotFoundException("Object not found in database");
	}
        
            
	aQueue.setId(Config.getCfg().tableToUniqueId(rs.getLong("sch_id"),
		    MaskQueue.class));
       

	//aQueue.setId(rs.getLong("sch_id"));
       
	aQueue.setOwnerId(rs.getInt("user_id"));

	aQueue.setName(rs.getString("item_name"));
	aQueue.setDescription(rs.getString("item_description").trim());
	aQueue.setStartTime(rs.getInt("item_start"));
	aQueue.setDuration(rs.getInt("item_duration"));
	aQueue.setTelescope(rs.getString("tel_code"));
	rs.close();
        QueueItem [] queueItems = QueueItemDBIO.read(stmt,id);
        QueueItem [] execQueueItems = QueueItemDBIO.readExecSeq(stmt,id);

        aQueue.setQueueList(queueItems);
        aQueue.setExecutionSequence(execQueueItems);
        aQueue.setSize(queueItems.length);
	TmpDBIO.deleteQueueReferenceData(aQueue.getId(),TMP_SCHEDULE_ITEMS,
					 stmt);
	TmpDBIO.copyQueueReferenceData(aQueue.getId(),obrepDbName_ + ".." + SCHEDULE_ITEMS,
				       TMP_SCHEDULE_ITEMS,stmt);


	TmpDBIO.deleteQueueReferenceData(aQueue.getId(),TMP_EXEC_SEQ,
					 stmt);
	TmpDBIO.copyQueueReferenceData(aQueue.getId(),obrepDbName_ + ".." + EXEC_SEQ,
				       TMP_EXEC_SEQ,stmt);	
	return obj;       
    }


    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
System.out.println("CLASS DbaseHandlerMaskQueue - write:");
	MaskQueue aQueue = (MaskQueue)obj.getObject();
	StringBuffer sb = new StringBuffer();

	sb.append(id);
	sb.append(",");
	sb.append(aQueue.getOwnerId());	
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(aQueue.getName()));
	sb.append(",");
	sb.append(SqlUtils.sqlEscapedString(aQueue.getDescription()));
	sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(aQueue.getTelescope()));
        sb.append(",");
        sb.append(SqlUtils.sqlEscapedString(MaskQueue.getType()));
	sb.append(",");
	sb.append(aQueue.getStartTime());
	sb.append(",");
	sb.append(aQueue.getDuration());	
        //sb.append("'");
	

	String sqlText = "INSERT INTO " + obrepDbName_ + ".." + SCHEDULES
		  + " ( " + obrepDbName_ + ".." + SCHEDULES + ".sch_id, "
	          + obrepDbName_ + ".." + SCHEDULES + ".user_id, " 
                  + obrepDbName_ + ".." + SCHEDULES + ".item_name, " 
	          + obrepDbName_ + ".." + SCHEDULES + ".item_description, " 	
	          + obrepDbName_ + ".." + SCHEDULES + ".tel_code, "
                  + obrepDbName_ + ".." + SCHEDULES + ".schedule_type, "
                  + obrepDbName_ + ".." + SCHEDULES + ".item_start, "
	          + obrepDbName_ + ".." + SCHEDULES + ".item_duration )"
		  + " VALUES (" + sb + ")";




	System.out.println("CLASS DbaseHandlerQueue - write: sqlText = "+sqlText);
	stmt.executeUpdate(sqlText);
	QueueItemDBIO.write(stmt,id,aQueue.getQueueList());
        QueueItemDBIO.writeExecSeq(stmt,id,aQueue.getExecutionSequence());
      }


    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {

	System.out.println("CLASS DbaseHandlerMaskQueue - update");
	delete(stmt, id);
	write(stmt, id, obj);
    }


    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id)
			throws SQLException {

      	
			  //	QueueItemDBIO.delete(stmt,id,TMP_SCHEDULE_ITEMS);
        QueueItemDBIO.delete(stmt,id,obrepDbName_ + ".." + EXEC_SEQ);
	QueueItemDBIO.delete(stmt,id,obrepDbName_ + ".." + SCHEDULE_ITEMS);
	String sqlString2 =  "DELETE FROM " + obrepDbName_ + ".." + DbaseHandlerQueue.SCHEDULES + 
			      " WHERE " + obrepDbName_ +  ".." + DbaseHandlerQueue.SCHEDULES + ".sch_id = " + id;
	    
	stmt.executeUpdate(sqlString2);

	System.out.println("After CLASS DbaseHandlerQueue - delete: sqlString = "+sqlString2);	    
    }
}

