package org.eso.ohs.persistence.dbase.phase2;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;

/**
 * I/O class for OBs stored in the database.
 *
 * <p>
 * $Id: DbaseHandlerOB.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */

public class DbaseHandlerOB extends DbaseHandlerCB {

    /**
     * Managed class refrs to dfs ObservationBlock.class
     * @return ObservationBlock class
     * 
     */

    public Class getManagedClass() {
        return ObservationBlock.class;
    }

    /**
     * Gets Observation block code "0"
     * @return "0"
     */

    protected String getType() {
        return CalibrationBlock.OBSBLK_TYPE;
    }

    /**
     * Reads additional Information that is not in Caliubration
     * block
     *
     * @param rs  Result set from read of obs_blocks table
     * @param cb  OB to be initialised with additional ObservationBlock info
     * @param vec Object Vector that initialises the extra objects that an OB
     *            refers to
     * @exception SQLException JDBC Exception occured reading the database
     */

    protected void readAdditional(
        ResultSet rs,
        CalibrationBlock cb,
        UnpackedStorableObject uso)
        throws SQLException {
        ObservationBlock ob = (ObservationBlock) cb;

        uso.addNestedObjId(
            "Target",
            Config.getCfg().tableToUniqueId(rs.getLong("tp_id"), Target.class));

        uso.addNestedObjId(
            "ConstraintSet",
            Config.getCfg().tableToUniqueId(
                rs.getLong("cs_id"),
                ConstraintSet.class));

        String val = rs.getString("calibrationReq");
        if (val != null) {
            val = val.trim();
        } else {
            val = "";
        }
        ob.setCalibrationRequirements(val);

        val = rs.getString("obs_grade");
        if (val != null) {
            val = val.trim();
        } else {
            val = "";
        }
        ob.setObsGrade(val);
    }

    /**
     * Gets Calibration requirements
     * @return calibration requirements 
     */
    protected String getCalibrationRequirements(CalibrationBlock cb) {
        ObservationBlock ob = (ObservationBlock) cb;
        return ob.getCalibrationRequirements();
    }

    /**
     * Gets ObsGrade value
     * @param cb reference to CalibrationBlock 
     * @return obsGrade value
     */
    protected String getObsGrade(CalibrationBlock cb) {
        ObservationBlock ob = (ObservationBlock) cb;
        return ob.getObsGrade();
    }

    /**
     * Return the dbase id to reference the attached TimeIntervals.
     * 
     * @param	cb  CalibrationBlock (or sublclass with TIs);
     */
    protected long getTimeIntId(CalibrationBlock cb) {
        ObservationBlock ob = (ObservationBlock) cb;
        TimeInterval[] timeInts = ob.getTimeIntervals();

        TimeInterval[] stTimeInts = ob.getSTTimeIntervals();
        if ((stTimeInts == null || stTimeInts.length == 0) &&
            (timeInts == null || timeInts.length == 0)) {
            return 0;
        }
        // XXX Could check if timeInt is the id 0 TimeInterval,
        //     ie from 0 seconds after Unix epoch until 2^32-1 secs
        // For the minute, keep it simple... :-)
        return Config.getCfg().uniqueToTableId(ob.getId());
    }
    
    /**
     * Return the dbase id to reference the attached TimeIntervals.
     * 
     * @param   cb  CalibrationBlock (or sublclass with TIs);
     */
    protected long getSTTimeIntId(CalibrationBlock cb) {
        ObservationBlock ob = (ObservationBlock) cb;
        TimeInterval[] timeInts = ob.getSTTimeIntervals();

        if (timeInts == null || timeInts.length == 0) {
            return 0;
        }

        // XXX Could check if timeInt is the id 0 TimeInterval,
        //     ie from 0 seconds after Unix epoch until 2^32-1 secs
        // For the minute, keep it simple... :-)

        return Config.getCfg().uniqueToTableId(ob.getId());
    }

}
