package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseHandlerBaseTG;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
import org.eso.ohs.persistence.dbase.phase1.Phase1SelectStmt;

/**
 * I/O class for ObsRun stored in the database.
 * 
 * <p>
 * $Id: DbaseHandlerOBTargetConstrainInfo.java,v 1.2 2005/08/05 16:48:51 pnunes
 * Exp $<br>
 * 
 * @author Bob Kemp
 * 
 *  
 */

public class DbaseHandlerOBTargetConstrainInfo extends DbaseIOHandler {

    private static final String STREHL_RATIO = "strehl_ratio";
    private static final String SKY_TRANSPARENCY = "sky_transparency";
    private static final String SEEING = "seeing";
    private static final String MOON_DISTANCE = "moon_distance";
    private static final String FLI = "fli";
    private static final String AIRMASS = "airmass";
    private static final String DIFFDEC = "diffdec";
    private static final String DIFFRA = "diffra";
    private static final String PROP_DEC = "propDec";
    private static final String PROP_RA = "propRA";
    private static final String DEC = "dec";
    private static final String RA = "ra";
    private static final String EXECUTION_TIME = "execution_time";
    private static final String OB_ID = "ob_id";
    private static final String TI_ID = "ti_id";
    private static final String CREATION_DATE = "creation_date";
    private static final String TYPE = "type";
    private static final String USER_PRIORITY = "userPriority";
    private static final String ITEM_NAME = "item_name";
    private static final String REPEATS = "repeats";
    private static final String STATUS = "status";
    private static final String RUN_ID = "run_id";
    private static final String USER_ID = "user_id";
    private static Logger stdlog_ = Logger
            .getLogger(DbaseHandlerOBTargetConstrainInfo.class);

    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerOBTargetConstrainInfo() {
    }

    /**
     * Managed class
     * 
     * @return PanelObsRun class
     */

    public Class getManagedClass() {
        return ObTargetConstraintInfo.class;
    }

    /**
     * Reads the object from the database, partially reassembling it. References
     * to other business objects are stored in other fields of the array
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be retrieved.
     * @return array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception IOException
     *                if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
            throws ObjectNotFoundException, IOException, SQLException {
        ResultSet rs = null;
        ObTargetConstraintInfo ob = (ObTargetConstraintInfo) newInstance();
        UnpackedStorableObject uso = new UnpackedStorableObject(ob);

        String sql = getSelectSql() + "WHERE "
                + Phase1SelectStmt.obsBlocksTable.get() + ".ob_id = " + id
                + "\n";

        stdlog_.debug(sql);
        rs = stmt.executeQuery(sql);

        if (!rs.next()) { // Only one with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        ob.setId(Config.getCfg().tableToUniqueId(id,
                ObTargetConstraintInfo.class));
        setObFields(ob, rs);

        rs.close();

        return uso;
    }

    /**
     * Read data
     * 
     * @param sql
     *            statement
     * @param whereClause
     *            where condition for retreiving the object
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        if (otherTables == null) {
            otherTables = "";
        }
        UnpackedStorableObject[] ob_array = null;
        ObTargetConstraintInfo ob = null;
        String sql = buildSqlQuery(whereClause, otherTables);
        stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Vector ob_vector = new Vector();

            while (rs.next()) {

                ob = (ObTargetConstraintInfo) newInstance();
                // New element
                ob_vector.addElement(ob);

                // Set only when attaching the object as a new
                // element
                ob.setId(Config.getCfg().tableToUniqueId(rs.getInt(OB_ID),
                        ObTargetConstraintInfo.class));
                setObFields(ob, rs);
            }
            rs.close();

            ob_array = new UnpackedStorableObject[ob_vector.size()];
            for (int i = 0; i < ob_vector.size(); i++) {
                ob = (ObTargetConstraintInfo) ob_vector.elementAt(i);
                ob_array[i] = new UnpackedStorableObject(ob);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return ob_array;
    }

    private String buildSqlQuery(String whereClause, String otherTables) {
    	
        String sql = getSelectSql() + otherTables + " WHERE " + getJoinClause()+ "\n AND "
                + whereClause + "\n ORDER BY "
                + Phase2SelectStmt.obsBlocksTable.get() + ".ob_id  ";
        return sql;
    }

    /**
     * Set the prog fields from the result set. Only for avoiding duplicated
     * code
     *  
     */
    private void setObFields(ObTargetConstraintInfo ob, ResultSet rs)
            throws SQLException {

        ob.setUserId(rs.getInt(USER_ID));
        ob.setRunId(rs.getInt(RUN_ID));
        ob.setStatus(rs.getString(STATUS));
        ob.setRepeats(rs.getInt(REPEATS));
        ob.setName(rs.getString(ITEM_NAME));
             ob.setUserPriority(rs.getInt(USER_PRIORITY));
        ob.setType(rs.getString(TYPE));
        ob.setCreationDate(rs.getLong(CREATION_DATE));
        ob.setTiId(rs.getLong(TI_ID));
        ob.setDbaseId(rs.getLong(OB_ID));
        ob.setExecutionTime(rs.getInt(EXECUTION_TIME));
        
        ob.setRa(rs.getInt(RA)/DbaseHandlerBaseTG.DEGREES_PER_HOUR);
        ob.setDec(rs.getInt(DEC));
        ob.setPropRA(rs.getInt(PROP_RA));
        ob.setPropDec(rs.getInt(PROP_DEC));
        ob.setDiffRA(rs.getFloat(DIFFRA));
        ob.setDiffDec(rs.getFloat(DIFFDEC));

        ob.setAirmass(rs.getFloat(AIRMASS));
        ob.setFractionalLunarIllumination(rs.getFloat(FLI));
        
        
        ob.setMoonAngularDistance(rs.getInt(MOON_DISTANCE) / 3600000);
        ob.setSeeing(rs.getFloat(SEEING));
        ob.setSkyTransparency(rs.getString(SKY_TRANSPARENCY));

        // don't know where it is in the database ??
        //ob.setTelTemp()
        //ob.setWaterVapour()

        ob.setStrehlRatio(rs.getFloat(STREHL_RATIO));

    }

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
        throw new ObjectIOException("Operation not supported for this object");
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, StorableObject obj) throws SQLException {
        throw new SQLException("Operation not supported for this object");
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject objSpec)
            throws SQLException {
        throw new SQLException("Operation not supported for this object");
    }

    /**
     * Deletes an object from the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @exception SQLException
     *                an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id) throws ObjectIOException {
        throw new ObjectIOException("Operation not supported for this object");
    }

    /**
     * @return Returns the run_sql.
     */
    private String getSelectSql() {
        String run_sql = "SELECT DISTINCT " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+OB_ID+",  " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+USER_ID+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+RUN_ID+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+REPEATS+", " + Phase2SelectStmt.obsBlocksTable.get()
                 + "."+ITEM_NAME+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+USER_PRIORITY+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+STATUS+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+TYPE+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+CREATION_DATE+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+EXECUTION_TIME+", " + Phase2SelectStmt.obsBlocksTable.get()
                + "."+TI_ID+", "

                + Phase2SelectStmt.constraintSetsTable.get() + "."+AIRMASS+", "
                + Phase2SelectStmt.constraintSetsTable.get() + "."+FLI+", "
                + Phase2SelectStmt.constraintSetsTable.get()
                + "."+MOON_DISTANCE+", "
                + Phase2SelectStmt.constraintSetsTable.get() + "."+SEEING+", "
                + Phase2SelectStmt.constraintSetsTable.get()
                + "."+SKY_TRANSPARENCY+", "
                + Phase2SelectStmt.constraintSetsTable.get()
                + "."+STREHL_RATIO+", "

                + Phase2SelectStmt.targetsTable.get() + "."+RA+", "
                + Phase2SelectStmt.targetsTable.get() + "."+DEC+", "
                + Phase2SelectStmt.targetsTable.get() + "."+PROP_RA+", "
                + Phase2SelectStmt.targetsTable.get() + "."+PROP_DEC+", "
                + Phase2SelectStmt.targetsTable.get() + "."+DIFFRA+", "
                + Phase2SelectStmt.targetsTable.get() + "."+DIFFDEC+" \n "

                + "FROM " + Phase2SelectStmt.obsBlocksTable.get() + ", "
                + Phase2SelectStmt.targetsTable.get() + ", "
                + Phase2SelectStmt.constraintSetsTable.get();
        return run_sql;
    }

    private String getJoinClause() {
        String res = Phase2SelectStmt.obsBlocksTable.get() + ".cs_id  = "
                + Phase2SelectStmt.constraintSetsTable.get() + ".cs_id  and "
                + Phase2SelectStmt.obsBlocksTable.get() + ".tp_id  = "
                + Phase2SelectStmt.targetsTable.get() + ".tp_id ";

        return (res);

    }

}