package org.eso.ohs.persistence.dbase.phase2;

import org.eso.ohs.core.dbb.sql.DbbSqlTable;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.dbase.phase1.Phase1SelectStmt;

/**
 * @author pnunes
 * 
 * Created on Aug 2, 2005
 * 
 */
public class Phase2SelectStmt {

    /**
     * Field obsBlockTable
     */
    public static DbbSqlTable obsBlocksTable = new DbbSqlTable("obs_blocks");

    /**
     * Field obsBlockTable
     */
    public static DbbSqlTable targetsTable = new DbbSqlTable("target_packages");

    /**
     * Field constraintSetTable
     */
    public static DbbSqlTable constraintSetsTable = new DbbSqlTable(
            "constraint_sets");

    /**
     * Field timeIntervalsTable
     */
    public static DbbSqlTable timeIntervalsTable = new DbbSqlTable(
            "time_intervals");

    /** table. */
    public final static DbbSqlTable usgFileTable = new DbbSqlTable("usg_file");

    /*
     * Field scheduleTable
     */
    public static DbbSqlTable scheduleTable = new DbbSqlTable("schedules");

    /*
     * Field scheduleTable
     */
    public static DbbSqlTable instTelTable = new DbbSqlTable("inst_at_tel ");

    /*
     * Field scheduleItemsTable
     */
    public static DbbSqlTable scheduleItemsTable = new DbbSqlTable(
            "schedule_items");

    public static DbbSqlTable runInfoViewTable = new DbbSqlTable("vw_run_info");

    /**
     * Method defineTableDb
     * 
     * @param dbName
     *            String
     * @param dbOwner
     *            String
     */
    public static void definePhase2TableDb(String dbName, String dbOwner) {
        DbbSqlTable[] tables =
        { obsBlocksTable, targetsTable, constraintSetsTable,
                timeIntervalsTable, scheduleTable, usgFileTable,
                scheduleItemsTable, runInfoViewTable, instTelTable };
        for (int i = 0; i < tables.length; i++) {
            tables[i].setDatabase(dbName);
            tables[i].setOwner(dbOwner);
        }
    }

    private static String inClause(long[] ids) {
        StringBuffer res = new StringBuffer();
        res.append(" in ( ");

        for (int i = 0; i < ids.length; i++) {
            long l = ids[i];
            if (i < (ids.length - 1)) {
                res.append(l).append(" ,");
            } else {
                res.append(l);
            }
        }
        res.append(" )");
        return res.toString();
    }

    /**
     * Returns a where condition for the in clause
     * 
     * @param tableIds
     * @return String
     */
    public static String wcObTargetConstraintInfoList(long[] tableIds) {
        StringBuffer stmt = new StringBuffer();

        stmt.append(obsBlocksTable.get()).append(".ob_id ").append(
                inClause(tableIds));
        return stmt.toString();
    }

    /**
     * Returns a where condition for joining obs_blocks with time_intervals
     * 
     * @param tableIds
     * @return long
     */
    public static String wcTimeConstraintInfoList(long[] tableIds) {
        StringBuffer stmt = new StringBuffer();

        stmt
                .append(timeIntervalsTable.get())
                .append(".ti_id = ")
                .append(obsBlocksTable.get())
                .append(".ob_id ")
                .append("\n AND ")
                .append(obsBlocksTable.get())
                .append(".ob_id ")
                .append(inClause(tableIds));
        return stmt.toString();
    }

    /**
     * Returns a where condition for joining obs_blocks with sched_rep in opc70
     * given a set of ob_ids
     * 
     * @param tableIds
     * @return long
     */
    public static String wcSchedRepObsBlocksList(long[] tableIds) {
        StringBuffer stmt = new StringBuffer();

        stmt
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".run_id = ")
                .append(obsBlocksTable.get())
                .append(".run_id ")
                .append("\n AND ")
                .append(obsBlocksTable.get())
                .append(".ob_id ")
                .append(inClause(tableIds));
        return stmt.toString();
    }

    /**
     * Returns a where condition for joining obs_blocks with sched_rep in opc70
     * given a where condition on obs_blocks table
     * 
     * @param tableIds
     * @return long
     */
    public static String wcSchedRepObsBlocksList(String whereCond) {
        StringBuffer stmt = new StringBuffer();

        stmt
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".run_id = ")
                .append(obsBlocksTable.get())
                .append(".run_id ")
                .append("\n AND ")
                .append(whereCond);
        return stmt.toString();
    }

    /**
     * Returns a where condition for joining obs_blocks with usg_file in obrep2
     * given a where condition on obs_blocks table
     * 
     * @param tableIds
     * @return long
     */
    public static String wcRunPropObsBlocksList(String whereCond) {
        StringBuffer stmt = new StringBuffer();

        stmt
                .append(Phase1SelectStmt.runsTable.get())
                .append(".id = ")
                .append(obsBlocksTable.get())
                .append(".run_id ")
                .append("\n AND ")
                .append(whereCond);
        return stmt.toString();
    }

    /**
     * @param tableIds
     * @return long
     */
    public static String wcObsBlocksTable() {
        StringBuffer stmt = new StringBuffer();
        stmt.append(", ").append(obsBlocksTable.get());
        return stmt.toString();
    }

    /**
     * @param tableIds
     * @return long
     */
    public static String wcScheduleTables(String tables) {
        StringBuffer stmt = new StringBuffer();
        stmt.append(tables).append(" , ").append(scheduleTable.get()).append(
                " , ").append(scheduleItemsTable.get());

        return stmt.toString();
    }

    /**
     * @return String
     */
    public static String includeJoinTimeIntervalObBlocks(String obsBlocksWhere) {
        StringBuffer stmt = new StringBuffer();
        stmt
                .append(timeIntervalsTable.get())
                .append(".ti_id = ")
                .append(obsBlocksTable.get())
                .append(".ob_id ")
                .append("\n AND ")
                .append(obsBlocksWhere);
        return stmt.toString();
    }

    /**
     * @param isSuperUser
     * @param user
     * @return String
     */
    public static String includeJoinSchedule(String[] schedNames, String user,
            boolean isSuperUser) {
        StringBuffer stmt = new StringBuffer();

        for (int i = 0; i < schedNames.length; i++) {
			schedNames[i] = TextUtils.quoteSqlString(schedNames[i]);
		}
        
        String scheNameList = TextUtils.arrayToString(schedNames, "','", true);
        stmt.append(scheduleTable.get()).append(".item_name IN ('").append(
                scheNameList).append("')").append(" \n AND ").append(
                obsBlocksTable.get()).append(".status NOT IN ('")
                .append(ObservationBlock.EXECUTED).append("','")
                .append(ObservationBlock.COMPLETED).append("')\n AND ");

        if (!isSuperUser) {
            stmt.append(addUserFilter(user, isSuperUser));
            stmt.append("\n AND ");
        }
        addScheduleJoin(stmt);
        return stmt.toString();
    }

    /**
     * @param isSuperUser
     * @param user
     * @return String
     */
    public static String includeJoinScheduleTelescope(String telescope,
            String user, boolean isSuperUser) {
        StringBuffer stmt = new StringBuffer();

        stmt.append(scheduleTable.get()).append(".tel_code = '").append(
                telescope).append("'\n AND ");
        if (!isSuperUser) {
            stmt.append(addUserFilter(user, isSuperUser));
            stmt.append("\n AND ");
        }
        addScheduleJoin(stmt);
        return stmt.toString();
    }

    /**
     * @param user
     * @param isSuperUser
     * @param stmt
     *            void
     */
    private static String addUserFilter(String user, boolean isSuperUser) {
        StringBuffer stmt = new StringBuffer();
        if (!isSuperUser) {
            stmt.append(scheduleTable.get()).append(".user_id = ").append(user);
        }
        return stmt.toString();
    }

    /**
     * @param stmt
     * @return
     */
    private static StringBuffer addScheduleJoin(StringBuffer stmt) {
        return stmt
                .append(scheduleTable.get())
                .append(".sch_id = ")
                .append(scheduleItemsTable.get())
                .append(".sch_id ")
                .append("\n AND ")
                .append(obsBlocksTable.get())
                .append(".ob_id = ")
                .append(scheduleItemsTable.get())
                .append(".ob_id ");
    }

    public static String selectQueues(String user, boolean isSuperUser) {
        StringBuffer stmt = new StringBuffer();

        stmt.append("select distinct ").append(scheduleTable.get()).append(
                ".item_name from ").append(scheduleTable.get());
        if (!isSuperUser) {
            stmt.append(" where ");
            stmt.append(addUserFilter(user, isSuperUser));
        }
        stmt.append("\n order by ").append(scheduleTable.get()).append(
        ".item_name");
        
        return stmt.toString();
    }

    /**
     * sql fro retrieving the current telescopes with todays queues.
     * 
     * @param instrumentList
     *            instruments for the today queues
     * @return String
     */
    public static String selectActiveTelInsAssociation(String[] instList,
            String endDate) {
        String inst = TextUtils.arrayToString(instList, "','", true);
        StringBuffer stmt = new StringBuffer();

        stmt
                .append("select distinct ")
                .append(instTelTable.get())
                .append(".tel_code, ")
                .append(instTelTable.get())
                .append(".inst_code from ")
                .append(instTelTable.get())
                .append("\n where ")
                .append(instTelTable.get())
                .append(".inst_code in ('")
                .append(inst)
                .append("')")
                .append(" \n AND ")
                .append(instTelTable.get())
                .append(".availability = 1")
                .append(" \n AND ")
                .append(instTelTable.get())
                .append(".end_date = '")
                .append(endDate)
                .append("'");

        return stmt.toString();
    }

    public static String selObStatus(long obId) {
        StringBuffer stmt = new StringBuffer();

        stmt
                .append("select  ")
                .append(obsBlocksTable.get())
                .append(".status ")
                .append("\n where ")
                .append(obsBlocksTable.get())
                .append(".ob_id = ")
                .append(obId);

        return stmt.toString();
    }

}