package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureFactory;
import org.eso.ohs.persistence.Config;

/**
 * I/O class for TemplateSignatures stored in the database.
 * 
 *
 * <p>
 * $Id: TemplateSigDBIO.java,v 1.1 2005/03/30 09:56:12 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 15-10-01 Add the obID to the errorList 
 *   of the Verify Report 
 */

public class TemplateSigDBIO {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(TemplateSigDBIO.class);

    private static String obrepDbName =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    /**
     * This method is called when BOB sends template Status Events back.
     * Thus updating the log table.
     * @param obId
     * @param tsNo
     * @param status
     * @throws SQLException
     */

    public static String getTemplateName(long obId, int tsNo)
        throws SQLException {
        Connection con = null;
        Statement stmt = null;
        Config cfg = Config.getCfg();
        SQLException error = null;
        String templateName = null;
        try {
            con =
                ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(),
                    cfg.getDbPassword());

            stmt = con.createStatement();
            templateName = getTemplateName(obId, tsNo, stmt);
        } catch (SQLException e) {
            error = e;
        }        
        finally {
            ConnectionPool.getPool().releaseConnection(con);
        }
        if(error != null)
             throw new SQLException(error.getMessage());
        if (templateName == null) {
            System.out.println("Template Name is Null");
            throw new IllegalArgumentException("No Template Exists on the database for this OB");
        }
        return templateName;
    }
    
    /**
     * Get the templateName where one has an OBID and 
     * @param obId
     * @param tsNo
     * @param stmt
     * @return String
     * @throws SQLException
     */

    private static String getTemplateName(long obId, int tsNo, Statement stmt)
        throws SQLException {
        String templateName = null;
        String sql =
            "SELECT "
                + obrepDbName
                + "..template_signatures.item_name"
                + " FROM "
                + obrepDbName
                + "..template_signatures "
                + "where "
                + obrepDbName
                + "..template_signatures.od_id = "
                + obId
                + " AND "
                + obrepDbName
                + "..template_signatures.ts_index = "
                + tsNo;
        stdlog_.debug(sql);        
        ResultSet rs = stmt.executeQuery(sql);
        while (rs.next()) {
            templateName = rs.getString("item_name");
        }
        return templateName;
    }

    /**    
     * Read template signature(s) from database
     * @param stmt Sql statement
     * @param instrumentName Instrument names associated with template
     * @param boID Id of ts(s) to be read
     * @exception SQLException
     *		    an error occurred during the dbase operation   
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * @exception ObjectIOException IO Eror occured while reading from the database
     *
     * @return array of template signatures
     */

    public static TemplateSignature[] read(
        Statement stmt,
        Instrument instrument,
        long boId)
        throws ObjectIOException, IOException, SQLException {

        Vector tsNames = new Vector();
        Vector vec = new Vector();
        ResultSet rs = null;

        String textLen =
            "SELECT max(datalength(long_value))"
                + " FROM"
                + " "
                + obrepDbName
                + "..template_signatures"
                + ","
                + obrepDbName
                + "..parameters"
                + " WHERE"
                + " "
                + obrepDbName
                + "..template_signatures.od_id = "
                + boId
                + " and "
                + obrepDbName
                + "..parameters.od_id = "
                + boId
                + " and "
                + obrepDbName
                + "..template_signatures.ts_index"
                + " = "
                + obrepDbName
                + "..parameters.ts_index";

        rs = stmt.executeQuery(textLen);
        int textLenMax = 0;
        while (rs.next()) {
            textLenMax = rs.getInt(1);
        }
        rs.close();

        stmt.execute("set TEXTSIZE " + textLenMax);

        String sqlText =
            "SELECT "
                + obrepDbName
                + "..template_signatures.ts_index AS tsSeq"
                + ", "
                + obrepDbName
                + "..template_signatures.item_name AS tsName"
                + ", "
                + obrepDbName
                + "..parameters.param_index AS paramSeq"
                + ", "
                + obrepDbName
                + "..parameters.item_name AS paramName"
                + ", "
                + obrepDbName
                + "..parameters.type AS paramType"
                + ", "
                + obrepDbName
                + "..parameters.value AS paramValue"
                + ", "
                + obrepDbName
                + "..parameters.long_value AS paramLongValue"
                + " FROM"
                + " "
                + obrepDbName
                + "..template_signatures"
                + ","
                + obrepDbName
                + "..parameters"
                + " WHERE "
                + obrepDbName
                + "..template_signatures.od_id = "
                + boId
                + " and "
                + obrepDbName
                + "..parameters.od_id = "
                + boId
                + " and "
                + obrepDbName
                + "..template_signatures.ts_index"
                + " = "
                + obrepDbName
                + "..parameters.ts_index";

        stdlog_.debug(sqlText);
        rs = stmt.executeQuery(sqlText);

        int tsNo, paramNo;

        while (rs.next()) {
            tsNo = rs.getInt("tsSeq");
            paramNo = rs.getInt("paramSeq");

            if (tsNo >= vec.size()) {
                int oldSize = vec.size();
                vec.setSize(tsNo + 1);
                for (int j = oldSize; j <= tsNo; j++) {
                    vec.setElementAt(new Hashtable(), j);
                }
                tsNames.setSize(tsNo + 1);
            }
            Hashtable paramValues = (Hashtable) vec.elementAt(tsNo);

            String name = rs.getString("paramName");
            String value = rs.getString("paramValue");
            tsNames.setElementAt(rs.getString("tsName"), tsNo);

            if (value == null) {
                value = rs.getString("paramLongValue");
            }

            if (value != null) {
                // A M Chavan, 05-Jan-01: removed trimming of value
                paramValues.put(name, value);
            }
        }
        rs.close();

        Vector tsVec = new Vector();
        int size = vec.size();

        for (int j = 0; j < size; j++) {
            Hashtable paramValues = (Hashtable) vec.elementAt(j);
            String tsf = (String) tsNames.elementAt(j);

            if (tsf == null) {
                tsVec.addElement(null);
                continue;
            }

            TemplateSignature ts =
                TemplateSignatureFactory.getTemplate(instrument, tsf);

            /** Template Signature is not null */
            if (ts != null) {
                Parameter[] params = ts.getParamList();
                for (int k = 0; k < params.length; k++) {
                    String value =
                        (String) paramValues.get(params[k].getParameterName());
                    if (value != null && !params[k].isFixed()) {
                        params[k].setValue(value);
                        if (params[k] instanceof ParamFileParameter) {
                            ParamFileParameter pfp =
                                (ParamFileParameter) params[k];
                            if (pfp.getFilename() == null) {
                                String filename = "unknown file";
                                int index = value.indexOf("PAF.NAME");
                                if (index >= 0) {
                                    String nameStr = value.substring(index);
                                    int start = nameStr.indexOf('"');
                                    int end;
                                    if (start >= 0) {
                                        end = nameStr.indexOf('"', start + 1);
                                        filename =
                                            nameStr.substring(start + 1, end);
                                    }
                                }
                                pfp.setFilename(filename);
                            }
                        }
                    }
                }
                tsVec.addElement(ts);
            }
            /** 
             *  cmavros 19-10-01
             *  WHen the Template Signature is null 
             *  Store all the missing or corrupt template names in errorListMsg vector
             *  If the error message for that template was not previously displayed
             *  throw exception to display error message.
             *  check if templateSignature message has previously been displayed 
             */
            else {
                String missing =
                    ":Template Signature File not found or invalid.\n";
                throw new ObjectIOException(tsf + missing);

            }

        } //End outermost for loop

        TemplateSignature[] tsList = new TemplateSignature[tsVec.size()];
        tsVec.copyInto(tsList);
        return tsList;
    }

    /**    
     * Write template signatures to the  database
     * @param stmt Sql statement
     * @param tsList List of template signatures to be written
     * @param isAcqTempl Is acquisition template
     * @exception SQLException
     *		    an error occurred during the dbase operation   
     */

    public static void write(
        Statement stmt,
        long boId,
        TemplateSignature[] tsList,
        boolean isAcqTempl)
        throws SQLException {
        StringBuffer sb = new StringBuffer();
        int ts_index = isAcqTempl ? 0 : 1; // Index as used in database.

        for (int tsNo = 0; tsNo < tsList.length; tsNo++, ts_index++) {
            Parameter[] params = tsList[tsNo].getParamList();

            sb.setLength(0);

            sb.append(boId);
            sb.append(",");
            sb.append(ts_index);
            sb.append(",");
            sb.append(
                SqlUtils.sqlEscapedString(tsList[tsNo].getTemplateName()));

            String sqlText =
                "INSERT INTO template_signatures"
                    + " (od_id, ts_index, item_name)"
                    + " VALUES ("
                    + sb
                    + ")";
            stdlog_.debug(sqlText);
            stmt.executeUpdate(sqlText);

            for (int paramNo = 0; paramNo < params.length; paramNo++) {
                sb.setLength(0);

                String name = params[paramNo].getParameterName();
                String type = String.valueOf(params[paramNo].getDBTypeCode());
                String value = params[paramNo].getValue();

                if (type.equals("p") && value.equals("NODEFAULT")) {
                    value = "";
                }

                sb.append(boId);
                sb.append(",");

                sb.append(ts_index);
                sb.append(",");

                sb.append(paramNo);
                sb.append(",");

                sb.append("'");
                sb.append(type);
                sb.append("'");
                sb.append(",");

                sb.append(getSubType(name));
                sb.append(",");

                sb.append(SqlUtils.sqlEscapedString(name));

                String extraFields = "";

                if (value != null) {
                    sb.append(",");
                    sb.append(SqlUtils.sqlEscapedString(value));

                    // If parameter value is too long, we
                    //  store the value in the the "long_value"
                    //  column, otherwise in "value" column.
                    // One of the columns will always be undefined.
                    extraFields = ", value";

                    if (value.length() > Config.getCfg().getMaxTextLength()) {
                        extraFields = ", long_value";
                    }

                }

                sqlText =
                    "INSERT INTO parameters"
                        + " (od_id, ts_index, param_index,"
                        + "  type, subtype, item_name"
                        + extraFields
                        + ")"
                        + " VALUES ("
                        + sb
                        + ")";
                //		stdlog_.debug(sqlText);
                stmt.executeUpdate(sqlText);
            }
        }
    }

    /**
     * Update template signature(s)
     * @param Sql statement
     * @param boId businiess object id for templates
     * @param acqTempl whether it is an caquisition template
     * @exception SQLException
     *		    an error occurred during the dbase operation 
     */

    public static void update(
        Statement stmt,
        long boId,
        TemplateSignature[] tsList,
        boolean isAcqTempl)
        throws SQLException {
        delete(stmt, boId, isAcqTempl);
        write(stmt, boId, tsList, isAcqTempl);
    }

    /**
     * Delete template signature(s)
     * @param Sql statement
     * @param boId businiess object id for templates
     * @param acqTempl whether it is an acquisition template
     * @exception SQLException
     *		    an error occurred during the dbase operation   
     */

    public static void delete(Statement stmt, long id, boolean acqTempl)
        throws SQLException {
        String selector = (acqTempl ? "ts_index = 0" : "ts_index != 0");
        String sqlText =
            "DELETE FROM template_signatures"
                + " WHERE od_id = "
                + id
                + " AND "
                + selector;
        stdlog_.debug(sqlText);
        stmt.executeUpdate(sqlText);

        sqlText =
            "DELETE FROM parameters"
                + " WHERE od_id = "
                + id
                + " AND "
                + selector;
        stdlog_.debug(sqlText);
        stmt.executeUpdate(sqlText);
    }

    /** <br>ChangeLog: A M Chavan, 01-Feb-2000: fixed bug: was not returning
     *                                  the correct subtype for grisms.
     * Get subtype depending on parameter
     * @param name parameter name 
     * @return subtype code
     */

    private static int getSubType(String name) {

        if (name.indexOf("FILT") >= 0) {
            return 1;
        }

        if (name.indexOf("GRIS") >= 0) {
            return 2;
        }

        return 0;
    }
}
