package org.eso.ohs.phase2.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * 
 * This class takes a given set of mask Ids and then 
 * dumps the associated adp/adm data out to a directory.
 * @author tcanavan
 *
 */
public class DumpMaskAdpAdmData extends ActionSuperclass {

    private ObjectContainer objectContainer_;

    private static SwingFileChooser fileChooser_ = new SwingFileChooser();
    static {
        File f1 = new File(AppConfig.getAppConfig().getParameterDir());
        fileChooser_.setCurrentDirectory(new File(f1.getParent()));
        fileChooser_.setSelectedFile(f1);
    }

    /**
     * Constructor for DumpAdpAdmData.
     */
    public DumpMaskAdpAdmData(ObjectContainer objectContainer,JFrame frm) {
        super(frm, "Dump Adp/Adm...");
        objectContainer_ = objectContainer;
    }
    
    /**
     * Constructor for DumpAdpAdmData.
     */
    public DumpMaskAdpAdmData(ObjectContainer objectContainer) {
        super((JComponent)objectContainer, "Dump Adp/Adm...");
        objectContainer_ = objectContainer;
    }


    /**
     * @see org.eso.ohs.core.gui.baseaction.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Long[] obIds = objectContainer_.getAllSelected();
        if (obIds == null || obIds.length <= 0) {
            JOptionPane.showMessageDialog(
                objectContainer_.getTopLevelAncestor(),
                new Object[] {
                    "There are no selected OB's",
                    " ",
                    "You must first select OB items in the list" },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        fileChooser_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        
        //fileChooser_.setCurrentDirectory(fileChooser_.getCurrentDirectory());

        int response =
            fileChooser_.showDialog(objectContainer_.getTopLevelAncestor(), "Select Dump Directory");
        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;
        if (response == JFileChooser.APPROVE_OPTION) {
            File f = fileChooser_.getSelectedFile();
            if (!f.canWrite()) {
                JOptionPane.showMessageDialog(
                objectContainer_.getTopLevelAncestor(),
                    new Object[] { "No write permissions to export to this directory" },
                    "No write permisssions",
                    JOptionPane.ERROR_MESSAGE);
            } else {
                new IncrementalActionAutomaton(
                    new IncrementalDump(obIds,f.getAbsolutePath()),
                    (JFrame) objectContainer_.getTopLevelAncestor(),
                    "Dumping Adp/Adm...");
            }

        }
    }

    public class IncrementalDump implements IncrementalAction {
        /** Array of ids of objects to be duplicated */
        private Long[] idLongs_;

        private Vector dumpVec = new Vector();
        
        private String dumpPath;

        /**
         * Constructor.
         *
         * @param device    media containing the objects to be duplicated
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalDump(Long[] idLongs,String path) {
            idLongs_ = idLongs;
            dumpPath = path;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        public void startOfAction() {
        }

        public void endOfAction() {
        }

        /**
         * Method to duplicate the n'th element of obs, where n = counter.
         *
         * @param counter counter to show which element of obs to check in.
         * @return true if the duplicating should continue
         *   and false otherwise.
         */
        public void performStep(int counter) {
            ObjectManager objMgr = ObjectManager.getObjectManager();
            Class objType =
                Config.getCfg().getClassFromId(idLongs_[counter].longValue());
            try {
                System.out.println("Object Type " + objType);
                CalibrationBlock cb =
                    (CalibrationBlock) objMgr.getBusObj(
                        Media.DBASE,
                        idLongs_[counter].longValue(),
                        objType);
                TemplateSignature ts = cb.firstTS();
                Parameter[] pList = ts.getParamList();
                for (int i = 0; i < pList.length; i++) {
                    if ((pList[i].getName().indexOf("INS.ADP") != -1)
                        || (pList[i].getName().indexOf("INS.ADM") != -1)) {
                        dumpFilename((ParamFileParameter) pList[i],dumpPath);
                        dumpVec.addElement(pList[i].getName());
                    }
                }
            } catch (ObjectNotFoundException e) {
                ErrorMessages.announceNoObject(
                    objectContainer_.getTopLevelAncestor(),
                    e);
            } catch (ObjectIOException e) {
                ErrorMessages.announceIOError(
                    objectContainer_.getTopLevelAncestor(),
                    e);
            } catch (IOException e) {
                ErrorMessages.announceIOError(
                    objectContainer_.getTopLevelAncestor(),
                    e);
            }
        }

    }

    /**
     * Dump files to  a directory.
     * @param param
     * @param directory
     * @throws IOException
     */

    public static void dumpFilename(ParamFileParameter param, String directory)
        throws IOException {
        String filename = ((ParamFileParameter) param).getFilename();
        if (filename != null && filename.length() > 0) {
            File outFile = new File(directory + File.separator + filename);
            FileOutputStream writeStream = new FileOutputStream(outFile);
            writeStream.write(param.getValue().getBytes());
            writeStream.close();
        }

    }

}
