package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.ImageFileView;
import org.eso.ohs.core.gui.widgets.ImageFilter;
import org.eso.ohs.core.gui.widgets.ImagePreview;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.core.utilities.ImageInfo;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.FindingChartSession;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AttachFindingChartsMultiOBSel extends ActionSuperclass {

    private static Logger stdlog_ =
        Logger.getLogger(AttachFindingChartsMultiOBSel.class);

    /**  
      * Static reference to file chooser dialog
      */

    private static SwingFileChooser chooser = new SwingFileChooser();

    /**
     * Static refrences to file chooser filters
     */

    private static ExtensionFileFilter jpegFilter =
        new ExtensionFileFilter("jpg", "Finding Charts");

    static {
        chooser.addChoosableFileFilter(new ImageFilter());
        chooser.setAcceptAllFileFilterUsed(false);

        //Add custom icons for file types.
        chooser.setFileView(new ImageFileView());

        //Add the preview pane.
        chooser.setAccessory(new ImagePreview(chooser));

        chooser.setMultiSelectionEnabled(true);
    }

    private Media device_;

    private FolderGridView folderGridView_;

    private ObservingRun obsRun;

    private String folderPath;

    private StringBuffer reportBuffer = new StringBuffer();

    private ObjectSelector objSelector_;

    private JFrame frm_;

    /**
     * Status window which displays reports on imports.     
     */

    private static TextDisplayWidget log_ =
        new TextDisplayWidget("Attach Finding Chart Report");

    /**
        * Constructs a create BusinessObject action.
        * 
        * @param device      medium containing the parent directory.
        * @param folderView  GUI view of (summary information in) the folder.
        */

    public AttachFindingChartsMultiOBSel(
        FolderGridView folderGridView,
        Media device) {
        super((JComponent) folderGridView, "Attach Finding Charts");
        folderGridView_ = folderGridView;
        device_ = device;
    }

    /**
        * Construct an AttachFindingCharts action.
        * 
        * @param device      medium containing the parent directy.
        * @param  folderView  GUI view of (summary information in) the folder
        */

    public AttachFindingChartsMultiOBSel(
        ObjectSelector objSelector,
        JFrame frm,
        Media device) {
        super(frm, "Attach Finding Charts");
        device_ = device;
        objSelector_ = objSelector;
        frm_ = frm;

    }

    /**
     * @see org.eso.ohs.core.gui.baseaction.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
        chooser.cancelSelection();
        int response = -1;
        ObservationBlock ob=null;
        if (folderGridView_ != null) {
            response =
                chooser.showDialog(
                    folderGridView_.getTopLevelAncestor(),
                    "Attach Finding Charts");
        } else {
            ob =
                (ObservationBlock) objSelector_.getDisplayedObject();
            response =
                chooser.showDialog(
                    frm_,
                    "Attach Finding Charts");

        }
        stdlog_.debug("Afrer response ");

        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;
        if (device_.equals(Media.LOCAL)) {
            if(folderGridView_ != null) {
                Folder folder = folderGridView_.getFolder().findRootFolder();
                DirectoryNode node = folder.getNode();
                obsRun = folderGridView_.getFolder().findObsRun();                
            } else {
                obsRun = ob.getObsRun();

            }
            Persistence persistence =
                (Persistence) Persistence.getObjectManager();
            try {
                folderPath = persistence.getRootFolderPath(obsRun);
                
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
            }
        }

        File[] files = chooser.getSelectedFiles();
        if (response == JFileChooser.APPROVE_OPTION) {
            try {
                //Object momento = MainWindow.getView().hideViews();
                if (objSelector_ == null) {
                    Long[] idLongs = folderGridView_.getAllSelected();
                    new IncrementalActionAutomaton(
                        new IncrementalAttach(
                            device_,
                            files,
                            chooser.getCurrentDirectory().getAbsolutePath(),
                            idLongs),
                        (MenuGroups) folderGridView_.getTopLevelAncestor(),
                        "Attaching...");

                } else {
                    stdlog_.debug("Before Attach");
                    Long[] idLong = new Long[1];
                    idLong[0] = new Long(ob.getId());
                    new IncrementalActionAutomaton(
                        new IncrementalAttach(
                            device_,
                            files,
                            chooser.getCurrentDirectory().getAbsolutePath(),
                            idLong),
                        frm_,
                        "Attaching...");
                }
            } catch (IOException e) {
                e.printStackTrace();
                announceIOError(e);
            } catch (ObjectIOException e) {
                e.printStackTrace();
                announceIOError(e);
            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
            }

        }
    }

    /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalAttach implements IncrementalAction {

        /** cached reference to MediaMgr */
        private ObjectManager mMgr_ = ObjectManager.getObjectManager();

        /** array of ids of objects to be duplicated */
        private Long[] idLongs_;

        /** Media containing the objects to be duplicated */
        private Media device_;

        private File[] files_;

        private String path_;

        private Vector findingChartVec = new Vector();

        private FindingChart[] fcs;

        private boolean cancelPressed = false;

        private int maxCharts;

        private int maxSize;

        private StringBuffer maxReportBuffer = new StringBuffer();

        private StringBuffer maxSizeReportBuffer = new StringBuffer();

        private StringBuffer findingChartSameChecksum = new StringBuffer();

        private StringBuffer findingChartSameName = new StringBuffer();
               
        private StringBuffer findingChartInvalidType = new StringBuffer();

        private StringBuffer alreadyCheckedIn = new StringBuffer();

        /**
         * Constructor.
         *
         * @param frm       window on which to base any dialog boxes.
         * @param device    media containing the objects to be duplicated
         * @param deepCopy  should we duplicate attached objects?
         * @param dir       directory containing the ObsBlocks
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */
        public IncrementalAttach(
            Media device,
            File[] files,
            String path,
            Long[] idLongs)
            throws IOException, ObjectNotFoundException, ObjectIOException {

            idLongs_ = idLongs;
            device_ = device;
            files_ = files;
        }

        /**
         * Number of steps in the action that are still remaining to be done.
         *
         * @return number of steps remaining (eg objects to acted upon)
         */
        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(idLongs_, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         *
         * @param stepNo        Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {
            if (idLongs_[counter] != null && !cancelPressed) {
                doAttach(idLongs_[counter].longValue());
            }
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {
            StringBuffer buf = new StringBuffer();
            log_.clearTextArea();
            if (maxReportBuffer.length() > 0) {
                buf.append(
                    "The following OB's could not have finding charts attached\n");
                buf.append(
                    "because the maximum of "
                        + maxCharts
                        + " has been exceeded\n");
                buf.append(maxReportBuffer.toString());
            }
            if (maxSizeReportBuffer.length() > 0) {
                buf.append(
                    "The following finding charts could not be attached\n");
                buf.append(
                    "because they exceeded the maximum file size of "
                        + maxSize);
                buf.append("\n");
                buf.append(maxSizeReportBuffer.toString());

            }

            if (alreadyCheckedIn.length() > 0) {
                buf.append(
                    "The following OB's could not have finding charts attached to them \n");
                buf.append("because they are already checked-in.\n");
                buf.append(alreadyCheckedIn.toString());
            }

            if (findingChartSameName.length() > 0) {
                buf.append(
                    "The following finding charts could not be attached \n");
                buf.append(
                    "because a finding chart with the same name already exists \n");
                buf.append(findingChartSameName.toString());
            }

            if (findingChartSameChecksum.length() > 0) {
                buf.append(
                    "The following finding charts could not be attached \n");
                buf.append(
                    "because a finding chart with the same contents but different name already exists \n");
                buf.append(findingChartSameChecksum.toString());
            }
            if (findingChartInvalidType.length() > 0) {
                buf.append(
                    "The following finding charts could not be attached \n");
                buf.append(
                    "because they are not in jpeg format \n");
                buf.append(findingChartInvalidType.toString());
            }

            if (buf.length() > 0) {
                log_.setTextArea(buf.toString());
                log_.setSize(new Dimension(700, 300));
                log_.setVisible(true);
            } else {
                log_.setVisible(false);
            }

        }

        /**
         * Utility method to duplicate a given business object
         *
         * @param id        id of business object to duplicate
         */
        public void doAttach(long id) {
            ObservationBlock ob = null;
            try {
                stdlog_.debug(">>>>>>>>>>>> ID " + id);
                ob =
                    (ObservationBlock) mMgr_.getBusObj(
                        device_,
                        id,
                        ObservationBlock.class);

            } catch (ObjectNotFoundException e) {
                e.printStackTrace();
                announceNoObject(e);
                return;
            } catch (ObjectIOException e) {
                e.printStackTrace();
                announceIOError(e);
                return;
            }
            if (ob.isCheckedIn()) {
                alreadyCheckedIn.append(ob.getName());
                alreadyCheckedIn.append("\n");
                return;
            }
            try {
                int currentFCs = ob.getFindingCharts().length;
                maxCharts = AppConfig.getAppConfig().getMaxFindingCharts();
                if (files_.length + currentFCs > maxCharts) {
                    maxReportBuffer.append(ob.getName());
                    maxReportBuffer.append("\n");
                    return;
                }
                maxSize = AppConfig.getAppConfig().getMaxFindingChartSize();
                Vector vec = new Vector();
                Hashtable fcCheckSumTable = new Hashtable();
                Hashtable fcNameTable = new Hashtable();

                for (int i = 0; i < files_.length; i++) {
                    String absPath = files_[i].getAbsolutePath();
                    stdlog_.debug("Absolute Path = " + absPath);
                    byte[] bData = FileIOUtils.readBinaryFile(absPath);
                    FileInputStream stream = new FileInputStream(absPath);
                    ImageInfo imageInfo = new ImageInfo();
                    imageInfo.setInput(stream);
                    if(!imageInfo.check() || imageInfo.getFormat()!=ImageInfo.FORMAT_JPEG) {
                        findingChartInvalidType.append(
                                                    "OB Name "
                                                        + ob.getName()
                                                        + " Finding Chart "
                                                        + files_[i].getName() + "\n");
                        continue;                                                                                   
                    }
                    if (bData.length > maxSize) {
                        maxSizeReportBuffer.append(
                            ob.getName() + " " + files_[i].getName());
                        maxSizeReportBuffer.append("\n");
                        continue;
                    }                    
                    FindingChart findingChart =
                        new FindingChart(
                            ob.getObsRun(),
                            bData,
                            files_[i].getName().trim(),
                            folderPath);

                    if (ob.hasFindingChartName(findingChart.getFcName())
                        || fcNameTable.containsKey(findingChart.getFcName())) {
                        findingChartSameName.append(
                            "OB Name "
                                + ob.getName()
                                + " Finding Chart "
                                + findingChart.getFcName());
                        findingChartSameName.append("\n");
                        continue;
                    }

                    if (ob.hasFindingChartChecksum(findingChart.getCheckSum())
                        || fcCheckSumTable.containsKey(
                            new Long(findingChart.getCheckSum()))) {
                        findingChartSameChecksum.append(
                            "OB Name "
                                + ob.getName()
                                + " Finding Chart "
                                + findingChart.getFcName());
                        findingChartSameChecksum.append("\n");
                        continue;
                    }
                    if (device_.equals(Media.DBASE)) {
                        findingChart.setFcData(bData);
                    } else {
                        stdlog_.debug("Write finding Chart");
                        findingChart.writeFindingChart(
                            findingChart.getCheckSum());
                    }
                    vec.addElement(findingChart);
                    fcCheckSumTable.put(
                        new Long(findingChart.getCheckSum()),
                        findingChart);
                    fcNameTable.put(findingChart.getFcName(), findingChart);
                }
                FindingChart[] fcCharts = new FindingChart[vec.size()];
                vec.copyInto(fcCharts);
                Vector attachFindingCharts = new Vector();
                FindingChartSession.appendFindingCharts(ob, fcCharts);

            } catch (IOException e) {
                e.printStackTrace();
                announceIOError(e);
            }

        }
    }
}
