package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JFrame;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.FCS;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.TIS;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.persistence.OHSPersistence;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.apps.p2pp.BOSelection;
import org.eso.ohs.phase2.apps.p2pp.LocalClipboard;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.PropertyClipboard;
import org.eso.ohs.phase2.apps.p2pp.views.ViewContainer;
/**
 * Action class to create a subdirectory of the selected directory.
 *
 * <p>
 * $Id: CopyPropertyAction.java,v 1.16 2005/11/17 11:22:10 tcanavan Exp $
 * <br>
 * Author: Bob Kemp
 */
public class CopyPropertyAction extends ActionSuperclass {
    
    

    private String property_;

    private ObjectSelector objSelector_;

    /*
       * Constructs a create directory action.
       * 
       * @param device   medium containing the parent directory.
       * @param path     path in HierarchyView tree to the selected directory.
       */
    public CopyPropertyAction(
        ObjectSelector objSelector,
        JFrame frm,
        String property,
        String action) {
        super(frm, action);

        property_ = property;
        objSelector_ = objSelector;
    }

    /**
     * Event handler for the create directory action, eg from a menu item.
     * 
     * @param ev       user-initiated ActionEvent to create a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        ViewContainer vc = MainWindow.view_.getOneWin();
        PropertyClipboard c = LocalClipboard.CLIPBOARD;
        BusinessObject bo = objSelector_.getDisplayedObject();

        if (bo instanceof ObservationBlock) {
            if (property_.equals("TemplateSignature")) {
                TemplateSignature ts =
                    new TemplateSignature(
                        ((ObservationBlock) bo).getTemplateSignature());
                BOSelection boSelection =
                    new BOSelection(ts, LocalClipboard.TEMPLATE_FLAVOR);
                c.setContents(boSelection, boSelection);

            }
            
            if (property_.equals("TimeIntervals")) {
                TimeInterval [] tis =
                        ((ObservationBlock) bo).getTimeIntervals();
                TIS ti =  new TIS(tis);        
                BOSelection boSelection =
                    new BOSelection(ti, LocalClipboard.TIS_FLAVOR);
                c.setContents(boSelection, boSelection);

            }
            
            if (property_.equals("STTimeIntervals")) {
                TimeInterval [] tis =
                        ((ObservationBlock) bo).getSTTimeIntervals();
                TIS ti =  new TIS(tis);        
                BOSelection boSelection =
                    new BOSelection(ti, LocalClipboard.ST_TIS_FLAVOR);
                c.setContents(boSelection, boSelection);

            }



            if (property_.equals("FindingCharts")) {
                FindingChart[] fcs = ((ObservationBlock) bo).getFindingCharts();
                OHSPersistence persistence = (OHSPersistence) OHSPersistence.getObjectManager();
                ObservationBlock ob = (ObservationBlock)bo;
                for(int i=0;i<fcs.length;i++) {
                	fcs[i].setObsRun(ob.getObsRun()); 
                }
                try {
                    String rootName = persistence.getRootFolderPath(ob.getObsRun());
                    FCS fcObject = new FCS(fcs,rootName);
                    BOSelection boSelection =
                        new BOSelection(fcObject, LocalClipboard.FCS_FLAVOR);
                    c.setContents(boSelection, boSelection);

                } catch (ObjectNotFoundException e) {                    
                    ErrorMessages.announceNoObject(null,e);
                    e.printStackTrace();
                    //ErrorMessages.announce
                }
            }

        }
        if (bo instanceof CalibrationBlock) {
            if (property_.equals("UserComments")) {
                String comment =
                    new String(((CalibrationBlock) bo).getUserComments());
                BOSelection selection =
                    new BOSelection(comment, LocalClipboard.OB_COMMENT_FLAVOR);
                c.setContents(selection, selection);
            }
        }
        if (bo instanceof ObservationBlock) {
            if (property_.equals("InstrumentComments")) {
                String comment =
                    new String(((ObservationBlock) bo).getInstrumentComments());
                BOSelection selection =
                    new BOSelection(comment, LocalClipboard.OB_COMMENT_INST_FLAVOR);
                c.setContents(selection, selection);
            }
        }

        c.firePropertyChangeEvent();
    }

    
}
