package org.eso.ohs.phase2.apps.p2ppAppServer;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * @author nkornwei
 *
 * A simple program to bind to a specified port and report when a connection to this
 * port is made. This is essentially to provide an easy means to test the ESO firewall.
 * Run this application as both the server and client on two different machines. The
 * client typically outside the ESO external fire wall.
 * Server:    -server <port>
 * Client:    -client <host/IP> <port>
 */

public class ConnectTest {

    /**
     * Constructor for ConnectTest.
     */
    public ConnectTest() {
        super();
    }

    public static void main(String[] args) {
        if (args.length == 0 || args[0].equals("-v")) {
            printUsage();
        }

        if (args[0].equals("-server")) {
            if (args.length < 2) {
                printUsage();
            }
            int port = -1;
            try {
                port = Integer.parseInt(args[1]);
            } catch (NumberFormatException nfe) {
                port = -1;
            }
            if (port <= 0) {
                System.out.println("Invalid port number: " + args[1]);
                printUsage();
            }
            try {
                ServerSocket s = new ServerSocket(port);
                System.out.println("Bound port " + port + " waiting....");
                while (true) {
                    Socket incomming = s.accept();
                    System.out.println("Got a connect and am closing it.");
                    incomming.close();
                }

            } catch (IOException e) {
                System.out.println("Error: " + e.toString());
                e.printStackTrace();
            }
        }
        if (args[0].equalsIgnoreCase("-client")) {
            if (args.length < 3) {
                printUsage();
            }
            int port = -1;
            try {
                port = Integer.parseInt(args[2]);
            } catch (NumberFormatException nfe) {
                port = -1;
            }
            if (port <= 0) {
                System.out.println("Invalid port number: " + args[2]);
                printUsage();
            }
            try {
                System.out.println(
                    "Connecting to " + args[1] + " port: " + port);
                Socket s = new Socket(args[1], port);
                System.out.println("Connected.");
                s.close();
            } catch (IOException e) {
                System.out.println("Error: " + e.toString());
                e.printStackTrace();
            }

        } else {
            printUsage();
        }
    }

    public static void printUsage() {
        System.out.println(
            "Usage "
                + ConnectTest.class.getName()
                + " -server <port to bind> | -client <host> <port>");
        System.exit(1);
    }
}
